#ifndef __ATTRIB_DLG_DB_H
#define __ATTRIB_DLG_DB_H
/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2001 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * attrib_dlg_db
 *
 * Synopsis:
 *   Routines used to interface the database from the attributes
 *   dialog.
 *
 * $Id: attrib_dlg_db.h,v 1.8 2001/11/17 22:36:50 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#include <glib.h>
#include <libpq-fe.h>

/*-------------------------------------------------------------------------
 * General Information:
 * 
 * o The read and delete functions all reaturn a pointer to the
 *   operation's result set.  This result can then be passed to
 *   other functions for data and status extraction.
 *   
 * o It is the user's responsibility to call PQclear (res) when
 *   the results of the query are no longer needed.  Failing to
 *   do this will introduce memory leaks.
 *------------------------------------------------------------------------
 */

/*-------------------------------------------------------------------------
 * Function: read_sevr_data
 *
 * Synopsis:
 *   Read the severiy data from the database.
 *
 * Inputs:
 *   no_inactive (boolean) : TRUE if inactive severities should be
 *                           suppressed in the output
 *   conn (PGconn *) : A pointer to the database connection.
 *
 * Outputs:
 *   Loads up static local data for output be other methods.
 * 
 * Return Value:
 *   Returns a pointer to the result set.
 *
 *-------------------------------------------------------------------------
 */
PGresult *read_sevr_data (PGconn *conn, gboolean no_inactive);


/*-------------------------------------------------------------------------
 * Function:  delete_severity
 *
 * Synopsis:
 *   Delete the severiry.
 *
 * Inputs:
 *   conn : pointer to the database connection
 *   pk_num : the PK number of the item to delete
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   Returns a pointer to the results of the operation.
 *
 *-------------------------------------------------------------------------
 */
PGresult *delete_severity (PGconn *conn, long pk_num);


/*-------------------------------------------------------------------------
 * Function: get_pk_num
 *
 * Synopsis:
 *   Get the pk number for the nth record of the given result set.
 *
 * Inputs:
 *   res     : pointer to the result set.
 *   n (int) : Index to the tuple to get (first tuple is 0).
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   The pk number (as a string) of the nth tuple.
 *
 *-------------------------------------------------------------------------
 */
const char *get_pk_num (PGresult *res, int n);

/*-------------------------------------------------------------------------
 * Function:  get_name
 *
 * Synopsis:
 *   Get the name for the nth tuple of the given result set.
 *
 * Inputs:
 *   res     : the result set.
 *   n (int) : Index to the tuple to get (first tuple is 0).
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   The name for the nth tuple, or an empty string if there is
 *   no nth tuple.
 *
 *-------------------------------------------------------------------------
 */
const char *get_name (PGresult *res, int n);

/*-------------------------------------------------------------------------
 * Function:  get_rank
 *
 * Synopsis:
 *   Get the rank for the given tuple.
 *
 * Inputs:
 *   res     : pointer to the result set.
 *   n (int) : Index to the tuple to get (first tuple is 0).
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   The rank for the nth tuple, or an empty string if there is
 *   no nth tuple.
 *
 *-------------------------------------------------------------------------
 */
const char *get_rank (PGresult *res, int n);

/*-------------------------------------------------------------------------
 * Function:  get_active_yn
 *
 * Synopsis:
 *   Get the active flag for the given tuple.
 *
 * Inputs:
 *   res     : pointer to the result set.
 *   n (int) : Index to the tuple to get (first tuple is 0).
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   "Yes" if the severity is active, no if it is not, or an empty 
 *   string if there is no nth tuple.
 *
 *-------------------------------------------------------------------------
 */
const char *get_active_yn (PGresult *res, int n);

/*-------------------------------------------------------------------------
 * Function:  get_descr
 *
 * Synopsis:
 *   Get the descrption for the given tuple.
 *
 * Inputs:
 *   res     : pointer to the result set.
 *   n (int) : Index to the tuple to get (first tuple is 0).
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   The descr for the nth tuple, or an empty string if there is
 *   no nth tuple.
 *
 *-------------------------------------------------------------------------
 */
const char *get_descr (PGresult *res, int n);

/*-------------------------------------------------------------------------
 * Function: read_stat_data
 *
 * Synopsis:
 *   Read the status data from the database.
 *
 * Inputs:
 *   no_inactive (boolean) : TRUE if inactive statuses should be
 *                           suppressed in the output
 *   conn (PGconn *) : A pointer to the database connection.
 *
 * Outputs:
 *   Loads up static local data for output by other methods.
 * 
 * Return Value:
 *   Returns a pointer to the result set.
 *
 *-------------------------------------------------------------------------
 */
PGresult *read_stat_data (PGconn *conn, gboolean no_inactive);


/*-------------------------------------------------------------------------
 * Function: delete_status
 *
 * Synopsis:
 *   Delete the given status.
 *
 * Inputs:
 *   conn : pointer to the database connection
 *   pk_num : PK number for the status to delete
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   Returns a pointer to the results of the operation.
 *
 *-------------------------------------------------------------------------
 */
PGresult *delete_status (PGconn *conn, long pk_num);

/*-------------------------------------------------------------------------
 * Function: read_type_data
 *
 * Synopsis:
 *   Read the type data from the database.
 *
 * Inputs:
 *   no_inactive (boolean) : TRUE if inactive types should be
 *                           suppressed in the output
 *   conn (PGconn *) : A pointer to the database connection.
 *
 * Outputs:
 *   Loads up static local data for output be other methods.
 * 
 * Return Value:
 *   Returns a pointer to the result set of the operation.
 *
 *-------------------------------------------------------------------------
 */
PGresult *read_type_data (PGconn *conn, gboolean no_inactive);


/*-------------------------------------------------------------------------
 * Function: delete_type
 *
 * Synopsis:
 *   Delete the given type.
 *
 * Inputs:
 *   conn : pointer to the database connection
 *   pk_num : PK number for the type to delete
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   Returns a pointer to the results of the operation.
 *
 *-------------------------------------------------------------------------
 */
PGresult *delete_type (PGconn *conn, long pk_num);

/*-------------------------------------------------------------------------
 * Function:  get_type_def_sevr
 *
 * Synopsis:
 *   Get the name of the default severity for the given tuple.
 *   This is only valid for results coming from a read of the
 *   "type" data.
 *
 * Inputs:
 *   res     : pointer to the result set.
 *   n (int) : Index to the tuple to get (first tuple is 0).
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   The default severity for the nth tuple, or an empty string if 
 *   there is no nth tuple.
 *
 *-------------------------------------------------------------------------
 */
const char *get_def_sevr (PGresult *res, int n);

#endif
