/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * find_dlg
 *
 * Synopsis:
 *   Find problem reports based on user specified criteria.
 *
 * $Id: find_dlg.c,v 1.31 2002/01/05 03:53:30 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <assert.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <gnome.h>
#include <libpq-fe.h>

#include <preps/gstr_utils.h>
#include <preps/pr_query.h>

#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "probupdate_dlg.h"
#include "util.h"

/*
 * Define the main widgets in the dialog box.
 */
#define CHANGE_LOG_CB        "change_log_cb"
#define DESCR_CB             "descr_cb"
#define FIX_CB               "fix_cb"
#define IGNORE_CASE_CB       "ignore_case_cb"
#define INACTIVE_CHECK_BOX   "inactive_cb"
#define PROBLEM_TYPE_CLIST   "problem_type_clist"
#define PROJECT_CLIST        "project_clist"
#define RESPONSIBLE_CLIST    "responsible_clist"
#define RESULTS_CLIST        "results_clist"
#define SEVERITY_CLIST       "severity_clist"
#define STATUS_CLIST         "status_clist"
#define SUBMITTER_CLIST      "submitter_clist"
#define TEXT_ENTRY           "text_entry"
#define TITLE_CB             "title_cb"

/*
 * The ORDER_BY_LIST is used to store ORDER BY information that the user
 * can change by selecting columns in the display list.  ORDER_BY_LIST
 * defaults to NULL and is reset to NULL when a new query is selected.
 * When ORDER_BY_LIST is NULL, the current query's ORDER BY is used.
 *
 * The elements in the ORDER_BY_LIST are in the same order that they
 * should appear in the ORDER BY clause. 
 *
 * The ORDER_BY_LIST is attached to the RESULTS_CLIST.
 */
#define ORDER_BY_LIST        "order_by_list"

#define PK_POS              0
#define NAME_POS            1
#define SELECT_PROJECT "\
SELECT project_num, name \
  FROM project prj \
 WHERE (EXISTS (SELECT * \
                  FROM submitter s \
                 WHERE s.project_num = prj.project_num \
                   AND s.login_id = getpgusername ()) \
     OR EXISTS (SELECT * \
                  FROM responsible r \
                 WHERE r.project_num = prj.project_num \
                   AND r.login_id = getpgusername ()) \
     OR prj.login_id = getpgusername ()) "
   
#define SELECT_STATUS "\
SELECT status_num, name, order_num \
  FROM status \
 WHERE 1 = 1 "

#define SELECT_PROBLEM_TYPE "\
SELECT type_num, name \
  FROM problem_type \
 WHERE 1 = 1 "

#define SELECT_SEVERITY "\
SELECT severity_num, name, order_num \
  FROM severity \
 WHERE 1 = 1 "

#define NO_INACTIVE       " AND active = TRUE "
#define ORDER_BY_NUM      " ORDER BY order_num "
#define SUB_RESP_ORDER_BY " ORDER BY 2, 1 "
#define ORDER_BY_NAME     " ORDER BY name "

/*
 * To weed out the potentially large number of submitters, only show
 * those who are in the submitter table for projects that are somehow
 * related to the current user.
 */
#define SELECT_SUBMITTER "\
SELECT login_id, \
       last_name || ', ' || first_name || ' (' || login_id || ')' \
  FROM person p \
 WHERE login_id in \
         (SELECT login_id \
            FROM submitter \
           WHERE project_num in (SELECT project_num \
                                   FROM project \
                                  WHERE login_id = getpgusername ()) \
              OR project_num in (SELECT project_num \
                                   FROM submitter \
                                  WHERE login_id = getpgusername ()) \
              OR project_num in (SELECT project_num \
                                   FROM responsible \
                                  WHERE login_id = getpgusername ())) " 

/*
 * See comment on SELECT_SUBMITTER.  Same applies here.
 */
#define SELECT_RESPONSIBLE "\
SELECT login_id, \
       last_name || ', ' || first_name || ' (' || login_id || ')' \
  FROM person p \
 WHERE login_id in \
         (SELECT login_id \
            FROM responsible \
           WHERE project_num in (SELECT project_num \
                                   FROM project \
                                  WHERE login_id = getpgusername ()) \
              OR project_num in (SELECT project_num \
                                   FROM submitter \
                                  WHERE login_id = getpgusername ()) \
              OR project_num in (SELECT project_num \
                                   FROM responsible \
                                  WHERE login_id = getpgusername ())) " 

#define PROJECT_ORDER_STRING        "prj.name" 
#define PROBLEM_TYPE_ORDER_STRING   "pt.name"
#define STATUS_ORDER_STRING         "stat.order_num"
#define SEVERITY_ORDER_STRING       "sevr.order_num"
#define MASTER_ORDER_STRING         "%s, %s, %s, %s, pr.problem_num "

/*
 * PR info display data
 *
 * This information also doubles for storing user defined ORDER BY info.
 * The scheme used in the order by list will be:
 *    (COL_NUM + 1) : order by that column
 *   -(COL_NUM + 1) : order by that column in descending order
 */
#define RESULTS_CLIST_COLS  6
#define PR_NUM_COL          0
#define PROJECT_COL         1
#define STATUS_COL          2
#define PROBLEM_TYPE_COL    3
#define SEVERITY_COL        4
#define TITLE_COL           5

/*
 * This application can only have one find dialog visable at a time, this
 * is the one...
 */
static GtkWidget  *find_dlg = NULL;

static void
raise_or_create_pr_update_dlg (GtkWidget *find_dialog,
                               PGconn    *conn,
                               gint       pk_num)
{
   GtkWidget  *main_win;  /* the application main window */
   GtkWidget  *upd_dlg;
   GList      *child_list;
   GList      *list_iter;

   main_win = gtk_object_get_data (GTK_OBJECT (find_dialog), MY_PARENT);
   g_assert (main_win != NULL);

   /*
    * Go through the existing list of children first.  If we have a child
    * that is already displaying this PR, just bring that one to the 
    * front.
    *
    * NOTE: All of the PR update dialogs are created as "children" of
    *       the main window, NONE are created as "children" of the find
    *       dialog.
    */
   child_list = gtk_object_get_data (GTK_OBJECT (main_win), upd_pr_list);
   for (list_iter = g_list_first (child_list);
        list_iter != NULL;
        list_iter = list_iter->next)
      {
      if (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                                PK_NUM)) == pk_num)
         {
         gdk_window_show (GTK_WIDGET (list_iter->data)->window);
         gdk_window_raise (GTK_WIDGET (list_iter->data)->window);

         return;
         }
      }

   /*
    * Didn't find a child already displaying the PR, so create one.
    */
   upd_dlg = create_problem_update_dlg (main_win, conn, pk_num);
   gtk_widget_show (upd_dlg);

   child_list = g_list_append (child_list, upd_dlg);
   gtk_object_set_data (GTK_OBJECT (main_win), upd_pr_list, child_list);

   gtk_signal_connect (GTK_OBJECT (upd_dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)upd_pr_list);

   return;
}


static void 
draw_results_clist (GtkCList  *clist, PGresult  *res)
{
   gint    i, n;
   gchar  *buff [RESULTS_CLIST_COLS]; 
   gint    pk;

   gtk_clist_freeze (clist);
   gtk_clist_clear (clist);
   n = PQntuples (res);
   for (i = 0; i < n; i++)
      {
      buff [PR_NUM_COL] = g_malloc
         (strlen (PQgetvalue (res, i, PR_NUMBER_POS)) + 1);
      strcpy (buff [PR_NUM_COL], PQgetvalue (res, i, PR_NUMBER_POS));

      buff [PROJECT_COL] = g_malloc 
         (strlen (PQgetvalue (res, i, PROJECT_NAME_POS)) + 1);
      strcpy (buff [PROJECT_COL], PQgetvalue (res, i, PROJECT_NAME_POS));

      buff [STATUS_COL] = g_malloc 
         (strlen (PQgetvalue (res, i, STATUS_NAME_POS)) + 1);
      strcpy (buff [STATUS_COL], PQgetvalue (res, i, STATUS_NAME_POS));

      buff [PROBLEM_TYPE_COL] = g_malloc 
         (strlen (PQgetvalue (res, i, PROBLEM_TYPE_NAME_POS)) + 1);
      strcpy (buff [PROBLEM_TYPE_COL], 
              PQgetvalue (res, i, PROBLEM_TYPE_NAME_POS));

      buff [SEVERITY_COL] = g_malloc 
         (strlen (PQgetvalue (res, i, SEVERITY_NAME_POS)) + 1);
      strcpy (buff [SEVERITY_COL], PQgetvalue (res, i, SEVERITY_NAME_POS));

      buff [TITLE_COL] = g_malloc 
         (strlen (PQgetvalue (res, i, PR_TITLE_POS)) + 1);
      strcpy (buff [TITLE_COL], PQgetvalue (res, i, PR_TITLE_POS));

      pk = atoi (PQgetvalue (res, i, PR_NUMBER_POS));
      
      gtk_clist_append (clist, buff);
      gtk_clist_set_row_data (clist, i, GINT_TO_POINTER (pk));

      g_free (buff [PR_NUM_COL]);
      g_free (buff [PROJECT_COL]);
      g_free (buff [STATUS_COL]);
      g_free (buff [PROBLEM_TYPE_COL]);
      g_free (buff [SEVERITY_COL]);
      g_free (buff [TITLE_COL]);
      }
   gtk_clist_thaw (clist);
}


static void 
fill_single_col_clist (GtkCList  *clist,
                       PGresult  *res,
                       gboolean   key_is_numeric)
{
   gint    i, n;
   gchar  *str_pk;
   gchar  *buff[1];

   gtk_clist_freeze (clist);
   gtk_clist_clear (clist);
   n = PQntuples (res);
   for (i = 0; i < n; i++)
      {
      buff[0] = g_malloc (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
      strcpy (buff[0], PQgetvalue (res, i, NAME_POS));
      gtk_clist_append (clist, buff);
      g_free (buff[0]);

      if (key_is_numeric)
         {
         gtk_clist_set_row_data 
            (clist, i, GINT_TO_POINTER (atoi (PQgetvalue (res, i, PK_POS))));
         }
      else
         {
         str_pk = g_malloc (strlen (PQgetvalue (res, i, PK_POS)) + 1);
         strcpy (str_pk, PQgetvalue (res, i, PK_POS));
         gtk_clist_set_row_data_full (clist, i, str_pk, g_free);
         }
      }
   gtk_clist_thaw (clist);

   return;
}

/*
 * draw_clists
 *
 * Fill the data in all of the selection clists (all clists except for the
 * results clist).
 *
 * Return TRUE if success, FALSE otherwise.
 */
static gboolean 
draw_clists (PGconn    *conn,
             GtkWidget *main_widget,
             gboolean   include_inactive)
{
   GtkCList    *current_clist;
   PGresult    *res;
   gboolean     in_error = FALSE;
   GString     *sql_buffer;

   /*
    * project clist
    */
   sql_buffer = g_string_new (SELECT_PROJECT);
   if (!include_inactive)
      {
      sql_buffer = g_string_append (sql_buffer, NO_INACTIVE);
      }
   g_string_append (sql_buffer, ORDER_BY_NAME);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Getting project data")))
      {
      current_clist = GTK_CLIST (get_widget (main_widget, PROJECT_CLIST));
      assert (current_clist);
      fill_single_col_clist (current_clist, res, TRUE);
      }
   else
      {
      in_error = TRUE;
      }
   PQclear (res);
   
   /*
    * status clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_STATUS);
      if (!include_inactive)
         {
         sql_buffer = g_string_append (sql_buffer, NO_INACTIVE);
         }
      sql_buffer = g_string_append (sql_buffer, ORDER_BY_NUM); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, _("Getting status data")))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, STATUS_CLIST));
         assert (current_clist);
         fill_single_col_clist (current_clist, res, TRUE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * problem type clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_PROBLEM_TYPE);
      if (!include_inactive)
         {
         sql_buffer = g_string_append (sql_buffer, NO_INACTIVE);
         }
      sql_buffer = g_string_append (sql_buffer, ORDER_BY_NUM); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, _("Getting problem type data")))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, 
                                                PROBLEM_TYPE_CLIST));
         assert (current_clist);
         fill_single_col_clist (current_clist, res, TRUE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * severity clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_SEVERITY);
      if (!include_inactive)
         {
         sql_buffer = g_string_append (sql_buffer, NO_INACTIVE);
         }
      sql_buffer = g_string_append (sql_buffer, ORDER_BY_NUM); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, _("Getting severity data")))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, SEVERITY_CLIST));
         assert (current_clist);
         fill_single_col_clist (current_clist, res, TRUE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /* 
    * submitter clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_SUBMITTER);
      if (!include_inactive)
         {
         sql_buffer = g_string_append (sql_buffer, NO_INACTIVE);
         }
      sql_buffer = g_string_append (sql_buffer, SUB_RESP_ORDER_BY); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, _("Getting submitter data")))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, SUBMITTER_CLIST));
         assert (current_clist);
         fill_single_col_clist (current_clist, res, FALSE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }
   
   /*
    * responsible clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_RESPONSIBLE);
      if (!include_inactive)
         {
         sql_buffer = g_string_append (sql_buffer, NO_INACTIVE);
         }
      sql_buffer = g_string_append (sql_buffer, SUB_RESP_ORDER_BY); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, _("Getting responsible data")))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, 
                                    RESPONSIBLE_CLIST));
         assert (current_clist);
         fill_single_col_clist (current_clist, res, FALSE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   return !in_error;
}


static void 
on_find_dlg_show (GtkWidget *window, gpointer  user_data)
{
   PGconn          *conn;
   GtkToggleButton *inactive_cb;
   GString         *config_str;
   gboolean         is_def;
   gboolean         flag;

   conn = gtk_object_get_data (GTK_OBJECT (window), MY_DB_CONN);
   g_assert (conn != NULL);

   inactive_cb = GTK_TOGGLE_BUTTON (get_widget (window, 
                                                INACTIVE_CHECK_BOX));
   g_assert (inactive_cb != NULL);

   config_str = g_string_new ("");

   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     FIND_DLG_SEC, SHOW_INACT, "false");
   flag = gnome_config_get_bool_with_default (config_str->str, &is_def);
   gtk_toggle_button_set_active (inactive_cb, flag);

   g_string_free (config_str, TRUE);

   if (!draw_clists (conn, window, flag))
      {
      gtk_widget_destroy (window);
      }

   return;
}


static void 
on_find_dlg_destroy (GtkWidget *window, gpointer  user_data)
{
   GtkCList        *clist;
   GList           *temp_list;
   GString         *config_str;
   GtkToggleButton *cb;

   config_str = g_string_new ("");

   cb = GTK_TOGGLE_BUTTON (get_widget (window, INACTIVE_CHECK_BOX));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     FIND_DLG_SEC, SHOW_INACT);
   gnome_config_set_bool (config_str->str, gtk_toggle_button_get_active (cb));

   g_string_sprintf (config_str, CONF_FILE, PACKAGE, DLG_INI_FILE);
   gnome_config_sync_file (config_str->str);

   g_string_free (config_str, TRUE);

   /*
    * If the user has selected any ordering, clean up the list that stores
    * the order by info...
    */
   clist = GTK_CLIST (get_widget (window, RESULTS_CLIST));
   assert (clist != NULL);
   temp_list = gtk_object_get_data (GTK_OBJECT (clist), ORDER_BY_LIST);
   if (temp_list)
      {
      gtk_object_remove_data (GTK_OBJECT (clist), ORDER_BY_LIST);
      g_list_free (temp_list);
      }

   return;
}


static gint
find_order_by_pos (const gchar *sql)
{
   gchar *curr;
   gint   diff;

   /*
    * If this is a properly formed query from libPRepS (and it should
    * be), there will be ONE and ONLY ONE "ORDER BY"...
    */
   curr = strstr (sql, "ORDER BY");
   g_assert (curr != NULL);

   diff = (gint)(curr - sql);
   return diff;
}


static void 
on_find_btn_clicked (GnomeDialog *dlg)
{
   pr_query_struct  *query;
   PGconn           *conn;
   GtkWidget        *main_widget;
   GtkCList         *current_clist;
   GList            *sel_list;
   PGresult         *res;
   gint              row_idx;
   gint              pk;
   gchar            *id;
   GString          *sql_buffer;
   GList            *order_by_list;
   GList            *list_iter;
   GString          *text_str;
   GtkEntry         *text_entry;
   gchar            *text_buf;
   GtkToggleButton  *cb;
   GString          *text_search_str = NULL;
   GString          *order_str = NULL;
   gchar             oper[5];
   gboolean          search_title;
   gboolean          search_descr;
   gboolean          search_fix;
   gboolean          search_change_log;
   gboolean          search_all;
   gboolean          add_or;

   /*
    * Setup any preliminary data that is needed.
    */
   main_widget = GTK_WIDGET (dlg);
   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   query = create_pr_query (PQuser (conn));

   /*
    * Get any project restrictions.
    */
   current_clist = GTK_CLIST (get_widget (main_widget, PROJECT_CLIST));
   assert (current_clist);
   sel_list = g_list_first (current_clist->selection);
   while (sel_list)
      {
      row_idx = GPOINTER_TO_INT (sel_list->data);
      pk = GPOINTER_TO_INT 
         (gtk_clist_get_row_data (current_clist, row_idx));
      add_project_restriction (query, pk);
      sel_list = sel_list->next;
      }

   /*
    * Get any status restrictions.
    */
   current_clist = GTK_CLIST (get_widget (main_widget, STATUS_CLIST));
   assert (current_clist);
   sel_list = g_list_first (current_clist->selection);
   while (sel_list)
      {
      row_idx = GPOINTER_TO_INT (sel_list->data);
      pk = GPOINTER_TO_INT 
         (gtk_clist_get_row_data (current_clist, row_idx));
      add_status_restriction (query, pk);
      sel_list = sel_list->next;
      }

   /*
    * Get any problem type restrictions.
    */
   current_clist = GTK_CLIST (get_widget (main_widget, PROBLEM_TYPE_CLIST));
   assert (current_clist);
   sel_list = g_list_first (current_clist->selection);
   while (sel_list)
      {
      row_idx = GPOINTER_TO_INT (sel_list->data);
      pk = GPOINTER_TO_INT
         (gtk_clist_get_row_data (current_clist, row_idx));
      add_problem_type_restriction (query, pk);
      sel_list = sel_list->next;
      }

   /* 
    * Get any severity restrictions.
    */
   current_clist = GTK_CLIST (get_widget (main_widget, SEVERITY_CLIST));
   assert (current_clist);
   sel_list = g_list_first (current_clist->selection);
   while (sel_list)
      {
      row_idx = GPOINTER_TO_INT (sel_list->data);
      pk = GPOINTER_TO_INT
         (gtk_clist_get_row_data (current_clist, row_idx));
      add_severity_restriction (query, pk);
      sel_list = sel_list->next;
      }

   /*
    * Get any submitter restrictions.
    */
   current_clist = GTK_CLIST (get_widget (main_widget, SUBMITTER_CLIST));
   assert (current_clist);
   sel_list = g_list_first (current_clist->selection);
   while (sel_list)
      {
      row_idx = GPOINTER_TO_INT (sel_list->data);
      id = gtk_clist_get_row_data (current_clist, row_idx);
      add_submitter_restriction (query, id);
      sel_list = sel_list->next;
      }

   /* 
    * Get any responsible restrictions.
    */
   current_clist = GTK_CLIST (get_widget (main_widget, RESPONSIBLE_CLIST));
   assert (current_clist);
   sel_list = g_list_first (current_clist->selection);
   while (sel_list)
      {
      row_idx = GPOINTER_TO_INT (sel_list->data);
      id = gtk_clist_get_row_data (current_clist, row_idx);
      add_responsible_restriction (query, id);
      sel_list = sel_list->next;
      }

   /*
    * Get any text seach restrictions
    */
   text_entry = GTK_ENTRY (get_widget (main_widget, TEXT_ENTRY));
   text_buf = gtk_entry_get_text (text_entry);
   if (strlen (text_buf) != 0)
      {
      text_str = g_string_new (text_buf);
      g_string_prepare_db_instr (text_str);

      cb = GTK_TOGGLE_BUTTON (get_widget (main_widget, TITLE_CB));
      search_title = gtk_toggle_button_get_active (cb);
      cb = GTK_TOGGLE_BUTTON (get_widget (main_widget, DESCR_CB));
      search_descr = gtk_toggle_button_get_active (cb);
      cb = GTK_TOGGLE_BUTTON (get_widget (main_widget, FIX_CB));
      search_fix = gtk_toggle_button_get_active (cb);
      cb = GTK_TOGGLE_BUTTON (get_widget (main_widget, CHANGE_LOG_CB));
      search_change_log = gtk_toggle_button_get_active (cb);
      /*
       * If they have none of them checked, just search them all...
       */
      search_all = !(search_title || search_descr || search_fix ||
                     search_change_log); 

      cb = GTK_TOGGLE_BUTTON (get_widget (main_widget, IGNORE_CASE_CB));
      g_assert (cb != NULL);
      if (gtk_toggle_button_get_active (cb))
         {
         strcpy (oper, " ~* ");
         }
      else
         {
         strcpy (oper, " ~ " );
         }

      text_search_str = g_string_new (" AND (");

      add_or = FALSE;
      if (search_title || search_all)
         {
         text_search_str = g_string_append (text_search_str, "pr.title");
         text_search_str = g_string_append (text_search_str, oper);
         text_search_str = g_string_append (text_search_str, text_str->str);
         add_or = TRUE;
         }

      if (search_descr || search_all)
         {
         if (add_or)
            {
            text_search_str = g_string_append (text_search_str, " OR ");
            }
         text_search_str = g_string_append (text_search_str, "pr.descr");
         text_search_str = g_string_append (text_search_str, oper);
         text_search_str = g_string_append (text_search_str, text_str->str);
         add_or = TRUE;
         }

      if (search_fix || search_all)
         {
         if (add_or)
            {
            text_search_str = g_string_append (text_search_str, " OR ");
            }
         text_search_str = g_string_append (text_search_str, "pr.fix");
         text_search_str = g_string_append (text_search_str, oper);
         text_search_str = g_string_append (text_search_str, text_str->str);
         add_or = TRUE;
         }

      if (search_change_log || search_all)
         {
         if (add_or)
            {
            text_search_str = g_string_append (text_search_str, " OR ");
            }
         text_search_str = g_string_append (text_search_str,
                                            "pr.change_log_text");
         text_search_str = g_string_append (text_search_str, oper);
         text_search_str = g_string_append (text_search_str, text_str->str);
         add_or = TRUE;
         }

      text_search_str = g_string_append (text_search_str, ") ");
      }

   /*
    * Build the order by...
    */
   current_clist = GTK_CLIST (get_widget (main_widget, RESULTS_CLIST));
   order_by_list = gtk_object_get_data (GTK_OBJECT (current_clist),
                                        ORDER_BY_LIST);
   if (order_by_list)
      {
      clear_order_by (query);
      order_str = g_string_new ("");
      list_iter = g_list_first (order_by_list);
      while (list_iter)
         {
         switch (abs (GPOINTER_TO_INT (list_iter->data)) - 1)
            {
            case PR_NUM_COL:
               order_str = g_string_append (order_str, "pr.problem_num");
               break;

            case PROJECT_COL:
               order_str = g_string_append (order_str, "prj.name");
               break;

            case STATUS_COL:
               order_str = g_string_append (order_str, "stat.name");
               break;

            case PROBLEM_TYPE_COL:
               order_str = g_string_append (order_str, "pt.name");
               break;

            case SEVERITY_COL:
               order_str = g_string_append (order_str, "sevr.name");
               break;

            case TITLE_COL:
               order_str = g_string_append (order_str, "pr.title");
               break;

            default:
               assert (FALSE);
            } /* end switch */

         if (GPOINTER_TO_INT (list_iter->data) < 0)
            {
            order_str = g_string_append (order_str, " desc");
            }
         list_iter = list_iter->next;
         if (list_iter)
            {
            order_str = g_string_append (order_str, ", ");
            }
         } /* end while */ 
      set_order_by (query, order_str->str);
      g_string_free (order_str, TRUE);
      } /* end if order_by_list */

   /*
    * Execute the query and display the results.
    */
   sql_buffer = create_query_string (query);
   if (text_search_str != NULL)
      {
      sql_buffer = g_string_insert (sql_buffer, 
                                    find_order_by_pos (sql_buffer->str), 
                                    text_search_str->str);
      g_string_free (text_search_str, TRUE);
      }
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Executing find query")))
      {
      draw_results_clist (current_clist, res);
      }
   destroy_pr_query (query);
   g_string_free (sql_buffer, TRUE);
   PQclear (res);
}


static void
on_results_click_col (GtkCList   *clist,
                      gint        column,
                      gpointer    user_data)
{
   GList     *list = NULL;
   gint       new_col = column + 1;

   list = gtk_object_get_data (GTK_OBJECT (clist), ORDER_BY_LIST);
   /*
    * The basic idea is:
    *   Make the current column the main "order by", retaining other
    *   columns further down in the list.
    *   If the current column already is the main "order by", invert
    *   the order for it.
    *   
    */
   if (list)
      {
      gtk_object_remove_data (GTK_OBJECT (clist), ORDER_BY_LIST);
      
      list = g_list_first (list);
      if (abs (GPOINTER_TO_INT (list->data)) == new_col)
         {
         new_col = -GPOINTER_TO_INT (list->data);
         }

      list = g_list_remove (list, GINT_TO_POINTER (new_col));
      list = g_list_remove (list, GINT_TO_POINTER (-new_col));
      }
   
   list = g_list_prepend (list, GINT_TO_POINTER (new_col));
   gtk_object_set_data (GTK_OBJECT (clist), ORDER_BY_LIST, list);

   /*
    * Act as if the "find" button has been clicked...
    */
   on_find_btn_clicked (user_data);
}

static void 
on_modify_btn_clicked (GnomeDialog *dlg)
{
   PGconn      *conn;
   GtkCList    *clist;
   gint         pk_num;
   GtkWidget   *msg_dlg;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), RESULTS_CLIST));
   assert (clist);

   /*
    * The user must have selected a PR to modify.
    */
   if (!clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You must select a problem to modify."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      return;
      }

   pk_num = GPOINTER_TO_INT
      (gtk_clist_get_row_data (clist, 
                               GPOINTER_TO_INT (clist->selection->data)));

   raise_or_create_pr_update_dlg (GTK_WIDGET (dlg), conn, pk_num);

   return;
}

static void
on_results_sel_row (GtkWidget      *widget,
                    gint            row,
                    gint            column,
                    GdkEventButton *event,
                    gpointer        user_data)                                  
{
   GtkCList   *clist = GTK_CLIST (widget);
   PGconn     *conn;
   gint        pk_num;

   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            pk_num = GPOINTER_TO_INT (gtk_clist_get_row_data (clist, row));
            conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
            g_assert (conn != NULL);

            raise_or_create_pr_update_dlg (GTK_WIDGET (user_data),
                                           conn, pk_num);
            break;

         default:
            break;
         }
      }
}

static void 
on_dismiss_btn_clicked (GnomeDialog *dlg)
{
   save_dlg_size (GTK_WIDGET (dlg), FIND_DLG_SEC, FALSE);
   gnome_dialog_close (dlg);
}


static void
on_mod_menu_item (GtkWidget *widget, gpointer user_data)
{
   on_modify_btn_clicked (GNOME_DIALOG (user_data));
}


static void
on_find_dlg_clicked (GnomeDialog *dlg,
                     gint         btn_cd,
                     gpointer     user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_find_btn_clicked (dlg);
         break;
      case 1:
         on_modify_btn_clicked (dlg);
         break;
      case 2:
         on_dismiss_btn_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached();
      }
}


static void 
on_inactive_cb_toggled (GtkToggleButton *togglebutton,
                        gpointer         user_data)
{
   GtkToggleButton  *inactive_cb;
   PGconn           *conn;

   inactive_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                INACTIVE_CHECK_BOX));

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   draw_clists (conn, GTK_WIDGET (user_data),
                gtk_toggle_button_get_active (inactive_cb));
}


static gboolean
on_button_press_event (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        user_data)
{
   gint       row;
   gint       col;
   gint       val;
   GtkWidget *popup;
   GtkWidget *main_win = GTK_WIDGET (user_data);

   static GnomeUIInfo popup_menu[] =
      {
      GNOMEUIINFO_ITEM_NONE (N_("_Edit Problem Report"),
                             N_("Edit the selected problem report"),
                             on_mod_menu_item),
      GNOMEUIINFO_END
      };

   if (event != NULL)
      {
      switch (event->button)
         {
         case 1:
         case 2:
            break;
         case 3:
            switch (event->type)
               {
               case GDK_BUTTON_PRESS:
                  val = gtk_clist_get_selection_info (GTK_CLIST (widget),
                                                      event->x,
                                                      event->y,
                                                      &row,
                                                      &col);
                  popup = gnome_popup_menu_new (popup_menu);
                  if (val)
                     {
                     gtk_clist_select_row (GTK_CLIST (widget), row, col);
                     }
                  else
                     {
                     gtk_widget_set_sensitive (popup_menu[0].widget, FALSE);
                     }
                  gnome_popup_menu_do_popup_modal (popup,
                                                   NULL,
                                                   NULL,
                                                   event,
                                                   main_win);
                  gtk_widget_destroy (popup);
                  break;
               default:
                  break;
               }
            break;
         default:
            break;
         }
      }

   return FALSE;
}


static GtkWidget *
create_find_dlg (GtkWidget *parent, PGconn  *conn)
{
  GtkWidget       *find_dlg;
  GtkWidget       *dialog_vbox1;
  GtkWidget       *vbox1;
  GtkWidget       *table1;
  GtkWidget       *label;
  GtkWidget       *sub_scrolledwindow;
  GtkWidget       *submitter_clist;
  GtkWidget       *resp_scrolledwindow;
  GtkWidget       *responsible_clist;
  GtkWidget       *prj_scrolledwindow;
  GtkWidget       *project_clist;
  GtkWidget       *stat_scrolledwindow;
  GtkWidget       *status_clist;
  GtkWidget       *pt_scrolledwindow;
  GtkWidget       *problem_type_clist;
  GtkWidget       *sevr_scrolledwindow;
  GtkWidget       *severity_clist;
  GtkWidget       *res_scrolledwindow;
  GtkWidget       *results_clist;
  GtkWidget       *num_lbl;
  GtkWidget       *project_lbl;
  GtkWidget       *status_lbl;
  GtkWidget       *problem_type_lbl;
  GtkWidget       *severity_lbl;
  GtkWidget       *title_lbl;
  GtkWidget       *inactive_cb;
  GtkWidget       *hbox;
  GtkWidget       *inner_hbox;
  GtkWidget       *text_lbl;
  GtkWidget       *text_entry;
  GtkWidget       *sep;
  GtkWidget       *cb;
  GtkTooltips     *tooltips;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  find_dlg = gnome_dialog_new (_("Find Problem Reports"),
                               NULL);
  gtk_container_border_width (GTK_CONTAINER (find_dlg), 5);
  GTK_WINDOW (find_dlg)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_policy (GTK_WINDOW (find_dlg), FALSE, TRUE, FALSE);
  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (find_dlg),
                                          _("Find"),
                                          GNOME_STOCK_PIXMAP_SEARCH);
  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (find_dlg),
                                          _("Modify"),
                                          GNOME_STOCK_PIXMAP_OPEN);
  gnome_dialog_append_button (GNOME_DIALOG (find_dlg),
                              GNOME_STOCK_BUTTON_CLOSE);
  gtk_signal_connect (GTK_OBJECT (find_dlg), "show",
                      GTK_SIGNAL_FUNC (on_find_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (find_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_find_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (find_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_find_dlg_clicked),
                      NULL);
  gtk_object_set_data (GTK_OBJECT (find_dlg), TOP_DOG, find_dlg);
  gtk_object_set_data (GTK_OBJECT (find_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (find_dlg), MY_DB_CONN, conn);
  set_dlg_size (find_dlg, FIND_DLG_SEC, 630, 400);

  dialog_vbox1 = GNOME_DIALOG (find_dlg)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  table1 = gtk_table_new (4, 4, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, FALSE, FALSE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 7);

  label = gtk_label_new (_("Status"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("Problem Type"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("Severity"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("Submitter"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 2, 3, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("Responsible"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 3, 4, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("Project"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  sub_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (sub_scrolledwindow);
  gtk_table_attach (GTK_TABLE (table1), sub_scrolledwindow, 2, 3, 1, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sub_scrolledwindow), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  submitter_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (find_dlg), 
                       SUBMITTER_CLIST, 
                       submitter_clist);
  gtk_widget_show (submitter_clist);
  gtk_container_add (GTK_CONTAINER (sub_scrolledwindow), submitter_clist);
  gtk_clist_set_column_width (GTK_CLIST (submitter_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (submitter_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (submitter_clist),
                                GTK_SELECTION_MULTIPLE);

  resp_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (resp_scrolledwindow);
  gtk_table_attach (GTK_TABLE (table1), resp_scrolledwindow, 3, 4, 1, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (resp_scrolledwindow), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  responsible_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (find_dlg), 
                       RESPONSIBLE_CLIST, responsible_clist);
  gtk_widget_show (responsible_clist);
  gtk_container_add (GTK_CONTAINER (resp_scrolledwindow), responsible_clist);
  gtk_clist_set_column_width (GTK_CLIST (responsible_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (responsible_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (responsible_clist),
                                GTK_SELECTION_MULTIPLE);

  prj_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (prj_scrolledwindow);
  gtk_table_attach (GTK_TABLE (table1), prj_scrolledwindow, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (prj_scrolledwindow), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  project_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (find_dlg), 
                       PROJECT_CLIST, project_clist);
  gtk_widget_show (project_clist);
  gtk_container_add (GTK_CONTAINER (prj_scrolledwindow), project_clist);
  gtk_clist_set_column_width (GTK_CLIST (project_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (project_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (project_clist),
                                GTK_SELECTION_MULTIPLE);

  stat_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (stat_scrolledwindow);
  gtk_table_attach (GTK_TABLE (table1), stat_scrolledwindow, 1, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (stat_scrolledwindow), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  status_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (find_dlg), 
                       STATUS_CLIST, status_clist);
  gtk_widget_show (status_clist);
  gtk_container_add (GTK_CONTAINER (stat_scrolledwindow), status_clist);
  gtk_clist_set_column_width (GTK_CLIST (status_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (status_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (status_clist),
                                GTK_SELECTION_MULTIPLE);

  pt_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (pt_scrolledwindow);
  gtk_table_attach (GTK_TABLE (table1), pt_scrolledwindow, 0, 1, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pt_scrolledwindow), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  problem_type_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (find_dlg), 
                       PROBLEM_TYPE_CLIST, 
                       problem_type_clist);
  gtk_widget_show (problem_type_clist);
  gtk_container_add (GTK_CONTAINER (pt_scrolledwindow), problem_type_clist);
  gtk_clist_set_column_width (GTK_CLIST (problem_type_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (problem_type_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (problem_type_clist),
                                GTK_SELECTION_MULTIPLE);

  sevr_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (sevr_scrolledwindow);
  gtk_table_attach (GTK_TABLE (table1), sevr_scrolledwindow, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sevr_scrolledwindow), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  severity_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (find_dlg),
                       SEVERITY_CLIST, 
                       severity_clist);
  gtk_widget_show (severity_clist);
  gtk_container_add (GTK_CONTAINER (sevr_scrolledwindow), severity_clist);
  gtk_clist_set_column_width (GTK_CLIST (severity_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (severity_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (severity_clist),
                                GTK_SELECTION_MULTIPLE);

  inactive_cb = gtk_check_button_new_with_label (_("Display inactive items"));
  gtk_object_set_data (GTK_OBJECT (find_dlg), 
                       INACTIVE_CHECK_BOX, 
                       inactive_cb);
  gtk_widget_show (inactive_cb);
  gtk_signal_connect (GTK_OBJECT (inactive_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_inactive_cb_toggled),
                      find_dlg);
  gtk_box_pack_start (GTK_BOX (vbox1), inactive_cb, FALSE, FALSE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (inactive_cb), FALSE);
  gtk_tooltips_set_tip 
     (tooltips, 
      inactive_cb,
      _("Check this box to display all statuses, problem types, severities, and projects, even if those items are inactive.  Otherwise only active items are shown."),
      NULL);

  sep = gtk_hseparator_new ();
  gtk_widget_show (sep);
  gtk_box_pack_start (GTK_BOX (vbox1), sep, FALSE, FALSE, 2);

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox, FALSE, FALSE, 0);

  inner_hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (inner_hbox);
  gtk_box_pack_start (GTK_BOX (hbox), inner_hbox, TRUE, TRUE, 0);

  text_lbl = gtk_label_new (_("Text:"));
  gtk_box_pack_start (GTK_BOX (inner_hbox), text_lbl, FALSE, FALSE, 0);
  gtk_widget_show (text_lbl);

  text_entry = gtk_entry_new();
  gtk_widget_ref (text_entry);
  gtk_object_set_data_full (GTK_OBJECT (find_dlg), TEXT_ENTRY, text_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (inner_hbox), text_entry, TRUE, TRUE, 0);
  gtk_widget_show (text_entry);

  cb = gtk_check_button_new_with_label (_("Ignore Case"));
  gtk_widget_ref (cb);
  gtk_object_set_data_full (GTK_OBJECT (find_dlg), IGNORE_CASE_CB, cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox), cb, FALSE, FALSE, 0);
  gtk_widget_show (cb);

  hbox = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox, FALSE, FALSE, 3);
  gtk_widget_show (hbox);

  text_lbl = gtk_label_new (_("Limit Text Search To:"));
  gtk_box_pack_start (GTK_BOX (hbox), text_lbl, FALSE, FALSE, 0);
  gtk_widget_show (text_lbl);

  cb = gtk_check_button_new_with_label (_("Title"));
  gtk_widget_ref (cb);
  gtk_object_set_data_full (GTK_OBJECT (find_dlg), TITLE_CB, cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox), cb, FALSE, FALSE, 0);
  gtk_widget_show (cb);

  cb = gtk_check_button_new_with_label (_("Description"));
  gtk_widget_ref (cb);
  gtk_object_set_data_full (GTK_OBJECT (find_dlg), DESCR_CB, cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox), cb, FALSE, FALSE, 0);
  gtk_widget_show (cb);

  cb = gtk_check_button_new_with_label (_("Fix Description"));
  gtk_widget_ref (cb);
  gtk_object_set_data_full (GTK_OBJECT (find_dlg), FIX_CB, cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox), cb, FALSE, FALSE, 0);
  gtk_widget_show (cb);

  cb = gtk_check_button_new_with_label (_("ChangeLog Description"));
  gtk_widget_ref (cb);
  gtk_object_set_data_full (GTK_OBJECT (find_dlg), CHANGE_LOG_CB, cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (hbox), cb, FALSE, FALSE, 0);
  gtk_widget_show (cb);

  sep = gtk_hseparator_new ();
  gtk_widget_show (sep);
  gtk_box_pack_start (GTK_BOX (vbox1), sep, FALSE, FALSE, 0);

  res_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (res_scrolledwindow);
  gtk_box_pack_start (GTK_BOX (vbox1), res_scrolledwindow, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (res_scrolledwindow), 5);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (res_scrolledwindow), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  results_clist = gtk_clist_new (RESULTS_CLIST_COLS);
  gtk_object_set_data (GTK_OBJECT (find_dlg), 
                       RESULTS_CLIST, 
                       results_clist);
  gtk_widget_show (results_clist);
  gtk_container_add (GTK_CONTAINER (res_scrolledwindow), results_clist);
  gtk_clist_set_column_width (GTK_CLIST (results_clist), PR_NUM_COL, 40);
  gtk_clist_set_column_width (GTK_CLIST (results_clist), PROJECT_COL, 80);
  gtk_clist_set_column_width (GTK_CLIST (results_clist), STATUS_COL, 80);
  gtk_clist_set_column_width (GTK_CLIST (results_clist), PROBLEM_TYPE_COL, 80);
  gtk_clist_set_column_width (GTK_CLIST (results_clist), SEVERITY_COL, 80);
  gtk_clist_column_titles_show (GTK_CLIST (results_clist));
  gtk_signal_connect (GTK_OBJECT (results_clist), "select_row",
                      GTK_SIGNAL_FUNC (on_results_sel_row),
                      find_dlg);
  gtk_signal_connect (GTK_OBJECT (results_clist), "click_column",
                      GTK_SIGNAL_FUNC (on_results_click_col),
                      find_dlg);
  gtk_signal_connect (GTK_OBJECT (results_clist), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press_event),
                      find_dlg);

  num_lbl = gtk_label_new (_("PR#"));
  gtk_widget_show (num_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (results_clist), 
                               PR_NUM_COL, num_lbl);

  project_lbl = gtk_label_new (_("Project"));
  gtk_widget_show (project_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (results_clist), 
                               PROJECT_COL, project_lbl);

  status_lbl = gtk_label_new (_("Status"));
  gtk_widget_show (status_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (results_clist), 
                               STATUS_COL, status_lbl);

  problem_type_lbl = gtk_label_new (_("Problem Type"));
  gtk_widget_show (problem_type_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (results_clist), 
                               PROBLEM_TYPE_COL, problem_type_lbl);

  severity_lbl = gtk_label_new (_("Severity"));
  gtk_widget_show (severity_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (results_clist), 
                               SEVERITY_COL, severity_lbl);

  title_lbl = gtk_label_new (_("Title"));
  gtk_widget_show (title_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (results_clist),
                               TITLE_COL, title_lbl);

  gtk_object_set_data (GTK_OBJECT (find_dlg), TOOLTIPS, tooltips);

  return find_dlg;
}


void
destroy_find_dlg (void)
{
   if (find_dlg != NULL)
      {
      gtk_widget_destroy (find_dlg);
      find_dlg = NULL;
      }
}


void
show_find_dlg (GtkWidget *parent, PGconn *conn)
{
   
   /*
    * If the dialog box exists, just make sure it is visable, otherwise
    * create it.
    */
   if (find_dlg)
      {
      gdk_window_show (find_dlg->window);
      gdk_window_raise (find_dlg->window);
      }
   else
      {
      find_dlg = create_find_dlg (parent, conn);
 
      /*
       * When the dialog is destroyed, call gtk_widget_destroyed(),
       * which will set the "user_data" (&find_dlg in this case) to
       * NULL.
       */
      gtk_signal_connect (GTK_OBJECT (find_dlg),
                          "destroy",
                          GTK_SIGNAL_FUNC (gtk_widget_destroyed),
                          &find_dlg);
      gtk_widget_show (find_dlg);
      }
}
