#ifndef __GTKUTILS_H
#define __GTKUTILS_H
/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2001 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * gtkutils
 *
 * Synopsis:
 *   Some of these utility functions are copied from the code 
 *   created by Glade.  This code is being placed here so that 
 *   I can create UI using Glade, but then split the code out 
 *   (something Glade does not currently fully support).
 *   
 *   Other functions and structures are used for passing data
 *   around within a set of widgets.
 *
 * $Id: gtkutils.h,v 1.15 2001/12/29 17:57:43 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */

#include <gtk/gtk.h>

/*
 * Common dialog box return values.
 */
#define ID_CANCEL             0
#define ID_OK                 1
#define ID_YES                2
#define ID_NO                 3

/*-------------------------------------------------------------------------
 * There is certain data that we will need to store with each high
 * level widget (a dialog box for example).  These definitions
 * provide keys for this data.  These keys should be used with the
 * gtk_object_[gs]et_data functions.
 *
 *  o TOP_DOG       : The "main widget" in a widget set (dialog,
 *                    windows, etc)
 *  o MY_PARENT     : My "parent", however it is defined by the
 *                    application.  Sets up a parent/child relationship
 *                    between top level widgets.
 *  o MY_DB_CONN    : My database connection (for db related apps)
 *  o TOOLTIPS      : The tooltips
 *
 *-------------------------------------------------------------------------
 */
#define TOP_DOG        "myself"
#define MY_PARENT      "my_parent"
#define MY_DB_CONN     "my_db_conn"
#define TOOLTIPS       "tooltips"

/*-------------------------------------------------------------------------
 * Function:  set_dirty
 *
 * Synopsis:
 *   Set the dirty flag for this widget.  This function most likely
 *   should be connected to whatever signal determines that this widget
 *   is now dirty.
 *
 *   This function does NOT do any before/after comparisons to ensure
 *   that the data actually IS dirty.
 *
 * Inputs:
 *   widget    : the widget to set dirty
 *   user_data : unused (here for Gtk compat)
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void set_dirty (GtkWidget *widget, gpointer user_data);

/*-------------------------------------------------------------------------
 * Function:  set_clean
 *
 * Synopsis:
 *   Clear the dirty flag for this widget.
 *
 * Inputs:
 *   widget    : the widget to set clean
 *   user_data : unused (here for Gtk compat)
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void set_clean (GtkWidget *widget, gpointer user_data);

/*-------------------------------------------------------------------------
 * Function:  set_all_clean
 *
 * Synopsis:
 *   Clear the dirty flag for all widgets in the parent's list of 
 *   widgets being tracked for clean/dirty status.
 *
 * Inputs:
 *   parent : the main widget, parent to all the widgets to set clean
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void set_all_clean (GtkWidget *parent);

/*-------------------------------------------------------------------------
 * Function:  add_widget_to_dirty_list
 *
 * Synopsis:
 *   Some top level widgets contain a list of child widgets that should
 *   be checked to see if they are dirty.  This function adds children
 *   to the list.
 *
 * Inputs:
 *   parent : the top level widget
 *   widget : the widget to add
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void add_widget_to_dirty_list (GtkWidget *parent, GtkWidget *widget);

/*-------------------------------------------------------------------------
 * Function:  destroy_dirty_list
 *
 * Synopsis:
 *   destroy the list of child widgets whose dirty status is being
 *   tracked
 *
 * Inputs:
 *   parent : the owner of the list
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void destroy_dirty_list (GtkWidget *parent);

/*-------------------------------------------------------------------------
 * Function:  widget_is_dirty
 *
 * Synopsis:
 *   Determine if the dirty flag has been set for the widget
 *
 * Inputs:
 *   widget : the widget to check
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   TRUE  : the widget is dirty
 *   FALSE : the widget is clean
 *
 *-------------------------------------------------------------------------
 */
gboolean widget_is_dirty (GtkWidget *widget);

/*-------------------------------------------------------------------------
 * Function:  widgets_children_are_dirty
 *
 * Synopsis:
 *   Called for a parent widget.  Checks all of the children in the 
 *   list of children being tracked, and returns TRUE if it finds
 *   at least one that is dirty.
 *
 * Inputs:
 *   parent : the owner of the list
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   TRUE  : at least one child is dirty
 *   FALSE : none of the children are dirty
 *
 *-------------------------------------------------------------------------
 */
gboolean widgets_children_are_dirty (GtkWidget *parent);


/*-------------------------------------------------------------------------
 * Function: list_dlg_has_dirty_child
 *
 * Synopsis:
 *   There are several dialogs that are "list" dialogs that can have 
 *   child edit dialogs.  This function is used to iterate the list
 *   of children and determine if any of the child edit dialogs are
 *   dirty.
 *
 * Inputs:
 *   dlg : pointer to the "list" type dialog.
 *   child_list_key : key to get the list of child edit dialogs.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   TRUE  : at least one child dlg is dirty
 *   FALSE : none of the children are dirty
 *
 *-------------------------------------------------------------------------
 */
gboolean list_dlg_has_dirty_child (GtkWidget *dlg, 
                                   const gchar *child_list_key);

/*-------------------------------------------------------------------------
 * Function:  child_dead
 *
 * Synopsis:
 *   In the child, link this handler to the "destroy" signal.  This
 *   will remove the child from the specified list (saved as object
 *   data in the parent).
 *
 *   Then, when the parent is destroyed, it can kill all remaining
 *   children by going through the current list.  Be sure to block 
 *   this handler first, or we could run into problems.
 *
 *   In order to use this function, the following MUST be true.
 *     o The child must save its parent as object data using
 *       MY_PARENT (defined above) as the key.
 *     o In the parent, the list the child is stored in must be
 *       object data accessed by the list name.
 *
 * Inputs:
 *   child_widget : the child widget
 *   list_name    : the name of the list (key of object data for list
 *                  in the parent widget)
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void child_dead (GtkObject *child_widget, gchar *list_name);

/*-------------------------------------------------------------------------
 * Function: gtkutils_cleanup
 *
 * Synopsis:
 *   This module keeps some data hanging around.  Clean it up.
 *   Calling funtions from this module after this function has
 *   been called will result in undefined behavior.
 *-------------------------------------------------------------------------
 */
void gtkutils_cleanup (void);

/*
 * Starting from here on down, this code is taken almost directly
 * from the code that Glade produces.
 */
/*-------------------------------------------------------------------------
 * Function:  get_widget
 *
 * Synopsis:
 *   This function returns a widget in a component created by Glade.
 *   Call it with the toplevel widget in the component (i.e. a 
 *   window/dialog), or alternatively any widget in the component, 
 *   and the name of the widget you want returned.
 *
 * Inputs:
 *   widget : pointer to the parent widget
 *   widget_name : the name of the child widget to find
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   A pointer to the widget if found, NULL otherwise.
 *
 *-------------------------------------------------------------------------
 */
GtkWidget *get_widget (const GtkWidget  *widget, 
                       const gchar      *widget_name);


/*-------------------------------------------------------------------------
 * Function:  set_notebook_tab
 *
 * Synopsis:
 *   This function is used by some of the generated code for the
 *   setting of notebook tabs.  Users probably will not need to 
 *   add calls to this function.
 *
 * Inputs:
 *   Not documented.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void set_notebook_tab (const GtkWidget  *notebook,
                       gint              page_num,
                       GtkWidget        *widget);


/*-------------------------------------------------------------------------
 * Function:  add_pixmap_directory
 *
 * Synopsis:
 *   Use this function to set the directory containing installed 
 *   pixmaps.
 *
 * Inputs:
 *   directory : dir to add (as a string). 
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
void add_pixmap_directory (const gchar *directory);


/*-------------------------------------------------------------------------
 * Function:  create_pixmap
 *
 * Synopsis:
 *   Used by the generated code to create pixmaps.  Users probably 
 *   will not need to call this function in their own code.
 *
 * Inputs:
 *   Not Documented.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *
 *-------------------------------------------------------------------------
 */
GtkWidget* create_pixmap (GtkWidget          *widget, 
                          const gchar        *filename);


/*-------------------------------------------------------------------------
 * Function:  create_arrow_button
 *
 * Synopsis:
 *   Create a button with an arrow in it.  
 *
 *   NOTE:  This code is lifted directly out of the GTK documentation.
 *
 * Inputs:
 *   arrow_type  : the type of arrow (up, down, left right)
 *   shadow_type : the type of shading (in, out, etched in, etched out)
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   Pointer to the arrow button.
 *
 *-------------------------------------------------------------------------
 */
GtkWidget *create_arrow_button (GtkArrowType  arrow_type,
                                GtkShadowType shadow_type);

#endif
