/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * mainwin
 *
 * Synopsis:
 *   Create and manipulate the main window for the application.
 *
 * $Id: mainwin.c,v 1.62 2002/01/20 02:50:19 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <libpq-fe.h>
#include <stdlib.h>
#include <syslog.h>

#include <preps/pr_query.h>
#include <props.h>

#include "about_dlg.h"
#include "attrib_dlg.h"
#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "find_dlg.h"
#include "gtkutils.h"
#include "login_dlg.h"
#include "person_dlg.h"
#include "prepsmain.h"
#include "print_dlg.h"
#include "prj_rpt_dlg.h"
#include "probentry_dlg.h"
#include "probupdate_dlg.h"
#include "project_dlg.h"
#include "query_dlg.h"
#include "util.h"

/*
 * Define the names of the significant widgets
 */
#define CLEAR_BUTTON         "clear_btn"
#define EXIT_BUTTON          "exit_button"
#define FIND_PR_BUTTON       "find_pr_button"
#define MAIN_WIN             "main_win"
#define MOD_PR_BUTTON        "mod_pr_button"
#define NEW_PR_BUTTON        "new_pr_button"
#define PROJECT_CLIST        "project_clist"
#define QUERY_COMBO          "query_combo"
#define QUERY_LIST           "query_list"
#define REFRESH_BUTTON       "refresh_button"
#define RESULTS_CLIST        "results_clist"
#define SHOW_INACTIVE_CB     "show_inactive_cb"
#define APPBAR               "appbar"

#define TIMEOUT              "timeout"
#define REFRESH_TICKS        "refresh_ticks"
#define REFRESH_COUNTER      "counter"
#define TICKS_PER_MIN        4
#define PERIOD_IN_MS         (60 / (TICKS_PER_MIN)) * 1000

/*
 * Status bar strings.
 */
#define DEFAULT_STAT_STRING _("Connected to database: %s")
#define NEW_STRING          _("Submit a new problem report")
#define EDIT_STRING         _("Edit the currently selected problem report")
#define FIND_STRING         _("Find specific problem reports")
#define PRINT_STRING        _("Print problem reports")
#define EXIT_STRING         _("Exit PRepS")
#define USER_QUERY_STRING   _("Create, edit, and delete user defined queries")
#define SYS_QUERY_STRING    _("Create, edit, and delete system wide queries")
#define PROJECTS_STRING     _("Manage the projects in this database")
#define USERS_STRING        _("Manage the users having access to this database")
#define ATTRIBUTES_STRING   _("Manage the defined severities, statuses, and \
problem types")
#define REFRESH_STRING      _("Re-execute the current query")
#define STAT_RPT_STRING     _("Generate the \"Problem Reports by Status\" \
Report")
#define VER_RPT_STRING      _("Generate the \"Problem Reports by Project Version\" Report")
#define CHANGE_LOG_STRING   _("Generate the \"Project ChangeLog\" Report") 

#define PK_POS        0
#define NAME_POS      1
#define SYS_FLAG_POS  2
#define PR_QUERY_QUERY "\
 SELECT query_num, name, is_system_query \
   FROM pr_query \
  WHERE is_system_query = TRUE \
     OR creator = getpgusername () \
  ORDER BY is_system_query, order_num, name"
 
#define PRJ_QUERY "\
 SELECT distinct prj.project_num, prj.name \
   FROM project prj \
  WHERE (prj.login_id = '%s' \
         OR exists (SELECT * \
                      FROM submitter s \
                     WHERE s.project_num = prj.project_num \
                       AND s.login_id = '%s') \
         OR exists (SELECT * \
                      FROM responsible r \
                     WHERE r.project_num = prj.project_num \
                       ANd r.login_id = '%s')) "
         
#define PRJ_ACTIVE " AND prj.active = TRUE "

#define PRJ_ORDER_BY " ORDER BY prj.name "
  
/*
 * The ORDER_BY_LIST is used to store ORDER BY information that the user
 * can change by selecting columns in the display list.  ORDER_BY_LIST
 * defaults to NULL and is reset to NULL when a new query is selected.
 * When ORDER_BY_LIST is NULL, the current query's ORDER BY is used.
 *
 * The elements in the ORDER_BY_LIST are in the same order that they
 * should appear in the ORDER BY clause. 
 *
 * The ORDER_BY_LIST is attached to the RESULTS_CLIST.
 */
#define ORDER_BY_LIST        "order_by_list"

/*
 * Keys for query related object data
 *
 * NUMBER_OF_COLS : key to the number of columns in a clist.  Stored
 *                  at the clist level
 * QUERY_COL_POS  : key to the position of the column within the pr_query.
 *                  Stored at the column widget level.
 * ORDER_COL_POS  : some columns need to be sorted by something other than
 *                  the contents of the column.  For example, for the status
 *                  we display the name, but sort by the status sort value.
 * MAX_RESULTS_COLS : The maximumn number of columns we can display in the
 *                    results clist.
 */
#define NUMBER_OF_COLS             "number_of_cols"
#define QUERY_COL_POS              "query_col_pos"
#define ORDER_COL_POS              "order_col_pos"
#define MAX_RESULTS_COLS           9

/*
 * define which columns should be displayed.
 */

/*
 * prototypes
 */
static void on_combo_sel_changed (GtkWidget  *widget, gpointer  user_data);
static void on_show_inactive_cb_toggled (GtkToggleButton *togglebutton,
                                         gpointer         user_data);


const gchar *upd_pr_list = "UPDATE_PR_LIST";
const gchar *new_pr_list = "NEW_PR_LIST";


static void
set_column_widget (GtkCList *clist, gint col, gint width,
                   const gchar *title, gint disp_col, gint order_col)
{
   GtkWidget  *lbl;

   if (width > 0)
      {
      gtk_clist_set_column_width (clist, col, width);
      }
   lbl = gtk_label_new (title);
   gtk_widget_show (lbl);
   gtk_clist_set_column_widget (clist, col, lbl);
   gtk_object_set_data (GTK_OBJECT (lbl), QUERY_COL_POS,
                        GINT_TO_POINTER (disp_col));
   gtk_object_set_data (GTK_OBJECT (lbl), ORDER_COL_POS,
                        GINT_TO_POINTER (order_col));
}

static GtkCList *
create_results_clist (void)
{
   gboolean   show_assignee;
   gboolean   show_problem_type;
   gboolean   show_prj_name;
   gboolean   show_pr_descr;
   gboolean   show_pr_num;
   gboolean   show_severity;
   gboolean   show_status;
   gboolean   show_submission_date;
   gboolean   show_submitter;
   gint       num_disp_cols = 0;
   gint       curr_col;
   GtkCList  *clist;

   show_assignee = display_column_in_results (ASSIGNEE_COL);
   if (show_assignee)
      {
      num_disp_cols++;
      }
   show_problem_type = display_column_in_results (PROBLEM_TYPE_COL);
   if (show_problem_type)
      {
      num_disp_cols++;
      }
   show_prj_name = display_column_in_results (PRJ_NAME_COL);
   if (show_prj_name)
      {
      num_disp_cols++;
      }
   show_pr_descr = display_column_in_results (PR_DESCR_COL);
   if (show_pr_descr)
      {
      num_disp_cols++;
      }
   show_pr_num = display_column_in_results (PR_RPT_NUM_COL);
   if (show_pr_num)
      {
      num_disp_cols++;
      }
   show_severity = display_column_in_results (SEVERITY_COL);
   if (show_severity)
      {
      num_disp_cols++;
      }
   show_status = display_column_in_results (STATUS_COL);
   if (show_status)
      {
      num_disp_cols++;
      }
   show_submission_date = display_column_in_results (SUBMISSION_DATE_COL);
   if (show_submission_date)
      {
      num_disp_cols++;
      }
   show_submitter = display_column_in_results (SUBMITTER_COL);
   if (show_submitter)
      {
      num_disp_cols++;
      }

   /*
    * The user has nothing checked.  This is not a good thing, so just
    * default to the descriptions...
    */
   if (num_disp_cols == 0)
      {
      show_pr_num = TRUE;
      show_pr_descr = TRUE;
      show_prj_name = TRUE;
      num_disp_cols = 3;
      }

   curr_col = 0;
   clist = GTK_CLIST (gtk_clist_new (num_disp_cols));
   gtk_object_set_data (GTK_OBJECT (clist), NUMBER_OF_COLS,
                        GINT_TO_POINTER (num_disp_cols));
   if (show_pr_num)
      {
      set_column_widget (clist, curr_col, 40, _("PR#"),
                         PR_NUMBER_POS, PR_NUMBER_POS);
      curr_col++;
      }
   if (show_prj_name)
      {
      set_column_widget (clist, curr_col, 80, _("Project"),
                         PROJECT_NAME_POS, PROJECT_NAME_POS);
      curr_col++;
      }
   if (show_status)
      {
      set_column_widget (clist, curr_col, 80, _("Status"),
                         STATUS_NAME_POS, STATUS_ORDER_POS);
      curr_col++;
      }
   if (show_problem_type)
      {
      set_column_widget (clist, curr_col, 80, _("Problem Type"),
                         PROBLEM_TYPE_NAME_POS, PROBLEM_TYPE_NAME_POS);
      curr_col++;
      }
   if (show_severity)
      {
      set_column_widget (clist, curr_col, 80, _("Severity"),
                         SEVERITY_NAME_POS, SEVERITY_ORDER_POS);
      curr_col++;
      }
   if (show_assignee)
      {
      set_column_widget (clist, curr_col, 80, _("Assignee"),
                         RESPONSIBLE_POS, RESPONSIBLE_POS);
      curr_col++;
      }
   if (show_submitter)
      {
      set_column_widget (clist, curr_col, 80, _("Submitter"),
                         SUBMITTER_POS, SUBMITTER_POS);
      curr_col++;
      }
   if (show_submission_date)
      {
      set_column_widget (clist, curr_col, 150, _("Submission Date"),
                         CREATION_DATE_POS, CREATION_DATE_POS);
      curr_col++;
      }
   if (show_pr_descr)
      {
      set_column_widget (clist, curr_col, -1, _("Title"),
                         PR_TITLE_POS, PR_TITLE_POS);
      curr_col++;
      }

   return clist;
}


static gboolean
dirty_children (GtkWidget *main_win)
{
   gboolean    dirty = FALSE;

   dirty = dirty || list_dlg_has_dirty_child (main_win, new_pr_list);
   dirty = dirty || list_dlg_has_dirty_child (main_win, upd_pr_list);
   dirty = dirty || attrib_dlg_dirty();
   dirty = dirty || person_dlg_dirty();
   dirty = dirty || project_dlg_dirty();
   dirty = dirty || query_dlg_dirty();

   return dirty;
}


static void
restore_sort_order (GtkWidget *main_win)
{
   GList    *list = NULL;
   GtkCList *clist;
   GString  *cfg;
   gboolean  is_def;
   gchar    *buffer;
   gchar    *curr;
   gchar    *next;

   /*
    * This clist, not the main_win, actually owns the sort order list.
    */
   clist = GTK_CLIST (get_widget (main_win, RESULTS_CLIST));
   g_assert (clist != NULL);

   cfg = g_string_new ("");
   g_string_sprintf (cfg, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     MAIN_WIN_SEC, "sort_order", "");

   buffer = g_strstrip
      (gnome_config_get_string_with_default (cfg->str, &is_def));
   if (!is_def && buffer[0] != '\0')
      {
      curr = buffer;
      next = strchr (curr, ':');
      while (curr != NULL)
         {
         list = g_list_append (list, GINT_TO_POINTER (atoi (curr)));
         curr = next;
         if (curr != NULL)
            {
            curr++;
            next = strchr (curr, ':');
            }
         }
      }

   gtk_object_set_data (GTK_OBJECT (clist), ORDER_BY_LIST, list);

   g_free (buffer);
   g_string_free (cfg, TRUE);
}

static void
save_sort_order (GList *list)
{
   GString *cfg;
   GList   *iter;
   GString *order;
   gchar    buffer[4];

   order = g_string_new ("");
   for (iter = g_list_first (list); iter != NULL; iter = iter->next)
      {
      g_assert (abs (GPOINTER_TO_INT (iter->data)) > 0 &&
                abs (GPOINTER_TO_INT (iter->data)) <= MAX_RESULTS_COLS);
      sprintf (buffer, "%d:", GPOINTER_TO_INT (iter->data));
      order = g_string_append (order, buffer);
      }
   /*
    * If we added anything, remove the last ':'
    */
   if (order->len > 0)
      {
      order = g_string_truncate (order, order->len - 1);
      }

   cfg = g_string_new ("");
   g_string_sprintf (cfg, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     MAIN_WIN_SEC, "sort_order");
   gnome_config_set_string (cfg->str, order->str);
   
   g_string_free (cfg, TRUE);
   g_string_free (order, TRUE);
}

/*
 *  The user can exit several ways.  This function centralizes the
 *  clean up.  Be sure to call it from each signal handler that is
 *  taking care of an exit request.
 */
static void 
cleanup_and_quit (GtkWidget  *main_win)
{
   GList     *temp_list;
   GList     *list_iter;
   GtkCombo  *combo;
   GtkCList  *clist;
   PGconn    *conn;
   gint       timer;
   GString   *cfg;
   gint       x;
   gint       y;
   
   cfg = g_string_new ("");
   gdk_window_get_position (main_win->window, &x, &y);
   g_string_sprintf (cfg, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     MAIN_WIN_SEC, X_POS);
   gnome_config_set_int (cfg->str, x);
   g_string_sprintf (cfg, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     MAIN_WIN_SEC, Y_POS);
   gnome_config_set_int (cfg->str, y);
                     
   gdk_window_get_size (main_win->window, &x, &y);
   g_string_sprintf (cfg, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     MAIN_WIN_SEC, X_SIZE);
   gnome_config_set_int (cfg->str, x);
   g_string_sprintf (cfg, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     MAIN_WIN_SEC, Y_SIZE);
   gnome_config_set_int (cfg->str, y);

   timer = GPOINTER_TO_INT
      (gtk_object_get_data (GTK_OBJECT (main_win), TIMEOUT));
   gtk_timeout_remove (timer);

   combo = GTK_COMBO (get_widget (main_win, QUERY_COMBO));
   g_assert (combo != NULL);
   temp_list = gtk_object_get_data (GTK_OBJECT (combo), QUERY_LIST);
   if (temp_list)
      { 
      destroy_name_pk_list (temp_list);
      }

   clist = GTK_CLIST (get_widget (main_win, RESULTS_CLIST));
   g_assert (clist != NULL);
   temp_list = gtk_object_get_data (GTK_OBJECT (clist), ORDER_BY_LIST);
   save_sort_order (temp_list);
   if (temp_list)
      {
      gtk_object_remove_data (GTK_OBJECT (clist), ORDER_BY_LIST);
      g_list_free (temp_list);
      }

   temp_list = gtk_object_get_data (GTK_OBJECT (main_win), upd_pr_list);
   if (temp_list)
      {
      list_iter = g_list_first (temp_list);
      while (list_iter)
         {
         gtk_signal_handler_block_by_func (GTK_OBJECT (list_iter->data),
                                           child_dead,
                                           (gpointer)upd_pr_list);
         gtk_widget_destroy (GTK_WIDGET (list_iter->data));
         list_iter = list_iter->next;
         }
      gtk_object_remove_data (GTK_OBJECT (main_win), upd_pr_list);
      g_list_free (temp_list);
      }

   conn = (PGconn *)gtk_object_get_data (GTK_OBJECT (main_win), MY_DB_CONN);
   if (conn != NULL)
      {
      db_shutdown (conn);
      }

   g_string_free (cfg, TRUE);
   gnome_config_sync();
   gtk_main_quit ();
}


/*
 * read_query_data
 * 
 * Read the query data from the database so we can put it
 * in our query list.
 */
static PGresult *
read_query_data (PGconn *conn)
{
   GString   *buffer;
   PGresult  *res;
   
   buffer = g_string_new (PR_QUERY_QUERY);
   res = PQexec (conn, buffer->str);
   
   g_string_free (buffer, TRUE);
   return res;
}


static void 
draw_pr_list (GtkWidget *parent,
              PGconn    *conn)
{
   GtkCList        *pr_clist;
   GtkCList        *prj_clist;
   GtkWidget       *widget;
   GtkCombo        *query_combo;
   GList           *query_list;
   GList           *sel_list;
   GList           *order_by_list;
   GList           *list_iter;
   GList           *cur_list_el;
   GList           *col_list = NULL;
   pr_query_struct *q;
   GString         *sql_buffer;
   GString         *new_order_str;
   PGresult        *res;
   gint             num_cols;
   gint             col;
   gint             n;
   gint             i;
   gint             idx;
   gint             pk;
   gint             save_pr_pk;
   gint             row_idx;
   gchar           *list_buf [MAX_RESULTS_COLS];
   gchar            num_buff[NUM_BUFFER_LEN];
   gboolean         first;
   
   /*
    * Get the widgets that we will need information from to build
    * the query, as well as the widget that will display the results.
    */
   pr_clist = GTK_CLIST (get_widget (parent, RESULTS_CLIST));
   prj_clist = GTK_CLIST (get_widget (parent, PROJECT_CLIST));
   query_combo = GTK_COMBO (get_widget (parent, QUERY_COMBO));
   g_assert (pr_clist != NULL);
   g_assert (prj_clist != NULL);
   g_assert (query_combo != NULL);
   /* 
    * query_list is not asserted, since there may be no queries set up,
    * in which case query_list will be NULL.
    */
   query_list = (GList *)gtk_object_get_data (GTK_OBJECT (query_combo), 
                                              QUERY_LIST);
   
   /*
    * We will need to clear out any PR data currently displayed.  Before
    * doing so, if there is a pr selected, get its PK num and save it.
    */
   save_pr_pk = -1;
   if (pr_clist->selection != NULL)
      {
      save_pr_pk = GPOINTER_TO_INT
         (gtk_clist_get_row_data
          (pr_clist,
           GPOINTER_TO_INT (pr_clist->selection->data)));
      }
   gtk_clist_freeze (pr_clist);
   gtk_clist_clear (pr_clist);
   
   /*
    * If there is a query selected (which there will be if there are
    * any queries to select), find out which one it is, and build
    * the query string.
    */
   if (query_list != NULL)
      {
      pk = find_pk_from_name 
         (query_list, gtk_entry_get_text (GTK_ENTRY (query_combo->entry)));
      g_assert (pk != INVALID_PK);
      q = create_pr_query_from_table (conn, PQuser (conn), pk);
      }
   else
      {
      q = create_pr_query (PQuser (conn));
      }
      
   /*
    * If any projects from the project list are chosen, add those 
    * limitations to the query.
    */
   if (prj_clist->selection != NULL)
      {
      sel_list = prj_clist->selection;
      while (sel_list != NULL)
         {
         row_idx = GPOINTER_TO_INT (sel_list->data);
         pk = GPOINTER_TO_INT (gtk_clist_get_row_data (prj_clist, 
                                                       row_idx));
         add_project_restriction (q, pk);
         sel_list = sel_list->next;
         }
      }

   num_cols = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (pr_clist),
                                                    NUMBER_OF_COLS));

   /*
    * If the user has clicked on any columns, order that way rather
    * than by the order of the query.
    */
   order_by_list = gtk_object_get_data (GTK_OBJECT (pr_clist), 
                                        ORDER_BY_LIST);
   if (order_by_list)
      {
      clear_order_by (q);
      new_order_str = g_string_new ("");
      list_iter = g_list_first (order_by_list);
      first = TRUE;
      while (list_iter)
         {
         col = abs (GPOINTER_TO_INT (list_iter->data)) - 1;
         /*
          * Sort orders are saved from session to session.  The user may
          * have chosen not to display some column they were previously
          * sorting on.  Thus cull out any column beyond what we are now
          * displaying in the window.
          */
         if (col >= num_cols)
            {
            cur_list_el = list_iter;
            list_iter = list_iter->next;
            order_by_list = g_list_remove_link (order_by_list, cur_list_el);
            g_list_free (cur_list_el);
            }
         else
            {
            if (!first)
               {
               new_order_str = g_string_append (new_order_str, ", ");
               }
            first = FALSE;
            widget = gtk_clist_get_column_widget (pr_clist, col);
         
            /*
             * The positions are zero indexed to be consistent with the
             * PQgetvalue () calls, but SQL is indexed at one.  Thus add
             * one to the value before making a string out of it for use
             * in the ORDER BY.
             */
            sprintf (num_buff, "%d",
                     GPOINTER_TO_INT (gtk_object_get_data
                        (GTK_OBJECT (widget),
                         ORDER_COL_POS)) + 1);
            new_order_str = g_string_append (new_order_str, num_buff);
            if (GPOINTER_TO_INT (list_iter->data) < 0)
               {
               new_order_str = g_string_append (new_order_str, " desc");
               }

            list_iter = list_iter->next;
            }
         } /* end while */
      set_order_by (q, new_order_str->str);
      g_string_free (new_order_str, TRUE);
      } /* end if order_by_list */
      
   /*
    * Get the query string, and execute the query.
    */
   sql_buffer = create_query_string (q);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, "Getting Problem Reports"))
      {
      /*
       * Each column widget contains the position within the pr_query 
       * of the data to be displayed by that column.  Rather than getting
       * this information for each row returned by the query, just get it
       * once, and store it in a list.
       */
      for (i = 0; i < num_cols; i++)
         {
         widget = gtk_clist_get_column_widget (pr_clist, i);
         col = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                     QUERY_COL_POS));
         col_list = g_list_append (col_list, GINT_TO_POINTER (col));
         }

      /*
       * Display the results.
       */
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         idx = 0;
         for (list_iter = g_list_first (col_list);
               list_iter != NULL;
               list_iter = list_iter->next)
            {
            col = GPOINTER_TO_INT (list_iter->data);
            list_buf[idx] = (char *)g_malloc
               (strlen (PQgetvalue (res, i, col)) + 1);
            strcpy (list_buf[idx], PQgetvalue (res, i, col));
            idx++;
            }
         g_assert (idx == num_cols);

         /*
          * Append the data to the list, and clean up the
          * list buffer.
          */
         gtk_clist_append (pr_clist, list_buf);
         pk = atoi (PQgetvalue (res, i, PR_NUMBER_POS));
         gtk_clist_set_row_data (pr_clist, i, GINT_TO_POINTER (pk));
         if (pk == save_pr_pk)
            {
            gtk_clist_select_row (pr_clist, i, 0);
            }

         for (idx = 0; idx < num_cols; idx++)
            {
            g_free (list_buf[idx]);
            }
         }
      g_list_free (col_list);
      }
   /*
    * Clean up before we leave.
    */
   PQclear (res);
   g_string_free (sql_buffer, TRUE);
   destroy_pr_query (q);
   gtk_clist_thaw (pr_clist);
}


static void 
draw_project_list (GtkWidget *parent,
                   PGconn    *conn)
{
   GtkCList        *prj_list;
   GtkCheckButton  *cb_widget;
   GString         *sql_buffer;
   PGresult        *res;
   gint             n;
   gint             i;
   gint             pk;
   gchar           *list_buf[1];
   
   /*
    * Get pointers to the widgets that we will need.
    */
   prj_list = GTK_CLIST (get_widget (parent, PROJECT_CLIST));
   cb_widget = GTK_CHECK_BUTTON (get_widget (parent, SHOW_INACTIVE_CB));
   
   /*
    * Build the project query.
    */
   sql_buffer = g_string_new ("");
   g_string_sprintf (sql_buffer, PRJ_QUERY, PQuser (conn),
                     PQuser (conn), PQuser (conn));
   /*
    * Limit to just active projects unless the "show inactve"
    * check box is checked.  (append the restriction if it is
    * not checked)
    */
   if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_widget)))
      {
      sql_buffer = g_string_append (sql_buffer, PRJ_ACTIVE);
      }

   sql_buffer = g_string_append (sql_buffer, PRJ_ORDER_BY);
   
   /*
    * Execute the query, and if all checks out, fill the project
    * list with the results.
    */
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, "filling project list"))
      {
      n = PQntuples (res);
      gtk_clist_freeze (prj_list);
      gtk_clist_clear (prj_list);
      for (i = 0; i < n; i++)
         {
         list_buf[0] = g_malloc 
            (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
         strcpy (list_buf[0], PQgetvalue (res, i, NAME_POS));
         gtk_clist_append (prj_list, list_buf);
         pk = atoi (PQgetvalue (res, i, PK_POS));
         gtk_clist_set_row_data (prj_list, i, GINT_TO_POINTER (pk));
         g_free (list_buf[0]);
         }
      gtk_clist_thaw (prj_list);
      }

   PQclear (res);
   g_string_free (sql_buffer, TRUE);
}


/*
 * Fill the combo box with the current user and system queries.  
 *
 * Attach a list to the combo that contains the PK/name pairs to 
 * make it easy to find the PK when the user picks a query.
 */
static void
draw_query_combo (GtkCombo *combo, PGconn   *conn)
{
   GList           *name_list = NULL;
   GList           *full_list = NULL; 
   PGresult        *res;
   gint             i, n;
   name_pk_struct  *curr_query;
   GString         *old_sel = NULL;
   gboolean         found = FALSE;

   /*
    * If there is an old list attached to the combo box, destroy it
    * before creating a new one.
    */
   full_list = gtk_object_get_data (GTK_OBJECT (combo), QUERY_LIST);
   if (full_list)
      { 
      /*
       * Save off the text for the currently selected item so we
       * can select it again after the list is rebuilt.
       */
      old_sel = g_string_new (gtk_entry_get_text (GTK_ENTRY (combo->entry)));
      gtk_object_remove_data (GTK_OBJECT (combo), QUERY_LIST);
      destroy_name_pk_list (full_list);
      full_list = NULL;
      }

   res = read_query_data (conn);
   if (chk_sql_error (res, "getting PR queries"))
      {
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         curr_query = (name_pk_struct *)g_malloc (sizeof (name_pk_struct));
         curr_query->pk = atoi (PQgetvalue (res, i, PK_POS));
         curr_query->name = (gchar *)g_malloc
            (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
         strcpy (curr_query->name, PQgetvalue (res, i, NAME_POS));
         /*
          * If an old selection to be restored was set, and we have not
          * found it yet, we need to see if this is the one.
          */
         if (old_sel && !found)
            {
            found = (strcmp (old_sel->str, curr_query->name) == 0);
            }
         name_list = g_list_append (name_list, curr_query->name);
         full_list = g_list_append (full_list, curr_query);
         }
      if (name_list)
         {
         gtk_combo_set_popdown_strings (combo, name_list);
         gtk_object_set_data (GTK_OBJECT (combo),
                              QUERY_LIST,
                              full_list);
         g_list_free (name_list); 
         }
      }
   PQclear (res);

   /*
    * If there was a previous selection, and we found it in the
    * new list, restore the combo to that selection.
    */
   if (old_sel)
      {
      if (found)
         {
         gtk_entry_set_text (GTK_ENTRY (combo->entry), old_sel->str);
         }
      g_string_free (old_sel, TRUE);
      }
}


void
refresh_query_combo (GtkWidget *main_win)
{
   GtkCombo  *query_combo;
   PGconn    *conn;

   query_combo =
      GTK_COMBO (get_widget (main_win, QUERY_COMBO));
   g_assert (query_combo != NULL);

   conn = gtk_object_get_data (GTK_OBJECT (main_win), MY_DB_CONN);
   g_assert (conn != NULL);

   draw_query_combo (query_combo, conn);
}


void
set_auto_refresh_time (GtkWidget *main_win)
{
   gint  target;

   target = auto_refresh_rate() * TICKS_PER_MIN;
   gtk_object_set_data (GTK_OBJECT (main_win), REFRESH_TICKS,
                        GINT_TO_POINTER (target)); 
}

static gint
periodic_processing (gpointer main_win)
{
   gint     refresh_count;
   gint     refresh_target;
   PGconn  *conn;

   refresh_target = GPOINTER_TO_INT
      (gtk_object_get_data (GTK_OBJECT (main_win), REFRESH_TICKS));
   if (refresh_target > 0)
      {
      refresh_count = GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (main_win), REFRESH_COUNTER)) + 1;
      if (refresh_count >= refresh_target)
         {
         refresh_count = 0;
         conn = gtk_object_get_data (GTK_OBJECT (main_win), MY_DB_CONN);
         draw_pr_list (GTK_WIDGET (main_win), conn);
         }
      gtk_object_set_data (GTK_OBJECT (main_win), 
                           REFRESH_COUNTER,
                           GINT_TO_POINTER (refresh_count));
      }

   return TRUE;
}


/*
 *  on_main_win_show
 *  
 *  Things that need to be done when we first show the main
 *  window:
 *    o Get the list of valid projects for the current
 *      user.  This is defined as the projects where the
 *      user is on the submitter list, the responsible list,
 *      or is the project leader.
 */
static void 
on_main_win_show (GtkWidget *widget,
                  gpointer   user_data)
{
   PGconn          *conn;
   GtkCombo        *query_combo;
   GString         *config_str;
   gchar           *name;
   gboolean         is_def;
   gboolean         flag;
   gint             pk;
   GList           *query_list;
   GtkToggleButton *cb;
   gint             timer;

   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   config_str = g_string_new ("");

   timer = gtk_timeout_add (PERIOD_IN_MS,
                            periodic_processing,
                            widget);
   gtk_object_set_data (GTK_OBJECT (widget), TIMEOUT, GINT_TO_POINTER (timer)); 
   restore_sort_order (widget);
   set_auto_refresh_time (widget);

   /* 
    * Set the query displayed in the query combo based on the INI file.
    * This will normally be the last one that the user had selected
    * last time preps was just down.
    */
   query_combo = GTK_COMBO (get_widget (widget, QUERY_COMBO));
   g_assert (query_combo != NULL);
   draw_query_combo (query_combo, conn);
   query_list = (GList *)gtk_object_get_data (GTK_OBJECT (query_combo), 
                                              QUERY_LIST);
   if (query_list)
      {
      g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DB_INI_FILE,
                        curr_db_name->str, MAIN_QUERY, "0");
      pk = gnome_config_get_int_with_default (config_str->str, &is_def);
      if (!is_def)
         {
         name = find_name_from_pk (query_list, pk);
         if (name != NULL)
            {
            gtk_entry_set_text (GTK_ENTRY (query_combo->entry), name);
            }
         }
      }

   /*
    * Set the show inactive toggle based on the INI file.
    */
   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DB_INI_FILE,
                     curr_db_name->str, MAIN_SH_INACT, "false");
   flag = gnome_config_get_bool_with_default (config_str->str, &is_def);
   cb = GTK_TOGGLE_BUTTON (get_widget (widget, SHOW_INACTIVE_CB));
   gtk_toggle_button_set_active (cb, flag);

   /*
    * No more config stuff beyond this point...
    */
   g_string_free (config_str, TRUE);

   draw_project_list (widget, conn);
   draw_pr_list (widget, conn);

   /* 
    * Now that everything is drawn, attach some of the handlers that
    * handle changes...
    */
  gtk_signal_connect (GTK_OBJECT (query_combo->entry), "changed",
                      GTK_SIGNAL_FUNC (on_combo_sel_changed),
                      widget);
  gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                      GTK_SIGNAL_FUNC (on_show_inactive_cb_toggled),
                      widget);
}


void 
refresh_project_list (gpointer data)
{
   PGconn *conn;

   conn = gtk_object_get_data (GTK_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);

   draw_project_list (GTK_WIDGET (data), conn);
}


void 
refresh_pr_list (gpointer data)
{
   PGconn  *conn;

   conn = gtk_object_get_data (GTK_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);

   draw_pr_list (GTK_WIDGET (data), conn);
}


static void
raise_or_create_pr_update_dlg (GtkWidget *main_win,
                               PGconn    *conn,
                               gint       pk_num)
{
   GtkWidget  *dlg;
   GList      *child_list;
   GList      *list_iter;

   /*
    * Go through the existing list of children first.  If we have a
    * child that already uses this PK, just bring that one to the 
    * front.
    */
   child_list = gtk_object_get_data (GTK_OBJECT (main_win), upd_pr_list);
   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      /*
       * If we find a child displaying the same PK, just raise it and
       * return.
       */
      if (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                                PK_NUM)) == pk_num)
         {
         gdk_window_show (GTK_WIDGET (list_iter->data)->window);
         gdk_window_raise (GTK_WIDGET (list_iter->data)->window);

         return;
         }
      list_iter = list_iter->next;
      }

   /*
    * Didn't find a child already displaying the PR, so create one.
    */
   dlg = create_problem_update_dlg (main_win, conn, pk_num);

   child_list = g_list_append (child_list, dlg);
   gtk_object_set_data (GTK_OBJECT (main_win), upd_pr_list, child_list);

   /*
    * Register child_dead() to remove the child from the list when it
    * dies.
    */
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)upd_pr_list);
   gtk_widget_show (dlg);

   return;
}


/*
 *  Signal handlers for the "Problem Reports" menu.
 *  
 *    New...
 *    Modify...
 *    Find...
 *    ---
 *    Exit
 */
static void 
on_new_pr (GtkWidget *widget, gpointer user_data)
{
   GtkWidget   *entry_dlg;
   PGconn      *conn;
   GList       *child_list;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   
   entry_dlg = create_problem_entry_dlg (GTK_WIDGET (user_data), conn);

   /*
    * Setup the child_dead() callback so the child is removed from the 
    * child list when the child dies.
    */
   gtk_signal_connect (GTK_OBJECT (entry_dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)new_pr_list);

   child_list = gtk_object_get_data (GTK_OBJECT (user_data), new_pr_list);
   child_list = g_list_append (child_list, entry_dlg);
   gtk_object_set_data (GTK_OBJECT (user_data), new_pr_list, child_list);
   
   gtk_widget_show (entry_dlg);
   
   return;
}


static void 
on_modify (GtkWidget  *widget, gpointer  user_data)
{
   PGconn      *conn;
   GtkCList    *pr_clist;
   gint         pk_num;
   GtkWidget   *msg_dlg;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   pr_clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), RESULTS_CLIST));
   g_assert (pr_clist != NULL);
   
   /*
    * The user must have selected a PR to modify.
    */
   if (!pr_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You must select a problem to modify."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      return;
      }
         
   pk_num = GPOINTER_TO_INT 
      (gtk_clist_get_row_data (pr_clist, 
                               GPOINTER_TO_INT (pr_clist->selection->data)));

   raise_or_create_pr_update_dlg (GTK_WIDGET (user_data), conn, pk_num);

   return;
}


static void 
on_find (GtkWidget *widget, gpointer user_data)
{
   PGconn   *conn;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   show_find_dlg (GTK_WIDGET (user_data), conn);
}


static void
on_login (GtkWidget *widget, gpointer user_data)
{
   GtkWidget   *msg_dlg;
   DbLoginData  db_data;
   gboolean     valid_new_db = FALSE;
   PGconn      *curr_conn;
   PGconn      *conn;
   GnomeAppBar *appbar;
   GString     *stat_string;
   GList       *temp_list;
   GList       *list_iter;

   /*
    * Check for dirty children.  If there are dirty children, and the
    * user chooses to cancel, just return without doing anything...
    */
   if (dirty_children (GTK_WIDGET (user_data)))
      {
      msg_dlg = gnome_message_box_new
         (_("Some items contain unsaved changes.\n"
            "If you continue, all unsaved changes will be lost.\n"
            "Do you want to continue?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 1)
         {
         return;
         }
      }

   curr_conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (curr_conn != NULL);
   init_db_data (&db_data);
   while (!valid_new_db &&
          execute_login_dlg (GTK_WIDGET (user_data), &db_data) == ID_OK)
      {
      conn = db_login (&db_data);
      if (PQstatus (conn) == CONNECTION_BAD)
         {
         syslog (LOG_ERR, "Error loggin on to database: %s",
                 db_data.db_name->str);
         msg_dlg = gnome_message_box_new (PQerrorMessage (conn),
                                          GNOME_MESSAGE_BOX_ERROR,
                                          GNOME_STOCK_BUTTON_OK,
                                          NULL);
         gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
         gnome_dialog_run (GNOME_DIALOG (msg_dlg));
         }
      else
         {
         /*
          * Any child windows we have will be associated with the old
          * database connection, so close them before shutting down the
          * old connection.
          */
         temp_list = gtk_object_get_data (GTK_OBJECT (user_data),
                                          (gpointer)upd_pr_list);
         if (temp_list != NULL)
            {
            list_iter = g_list_first (temp_list);
            while (list_iter)
               {
               gtk_signal_handler_block_by_func (GTK_OBJECT (list_iter->data),
                                                 child_dead, 
                                                 (gpointer)upd_pr_list);
               gtk_widget_destroy (GTK_WIDGET (list_iter->data));
               list_iter = list_iter->next;
               }
            g_list_free (temp_list);
            gtk_object_remove_data (GTK_OBJECT (user_data), upd_pr_list);
            }
         temp_list = gtk_object_get_data (GTK_OBJECT (user_data),
                                          (gpointer)new_pr_list);
         if (temp_list != NULL)
            {
            list_iter = g_list_first (temp_list);
            while (list_iter != NULL)
               {
               gtk_signal_handler_block_by_func (GTK_OBJECT (list_iter->data),
                                                 child_dead,
                                                 (gpointer)new_pr_list);
               gtk_widget_destroy (GTK_WIDGET (list_iter->data));
               list_iter = list_iter->next;
               }
            g_list_free (temp_list);
            gtk_object_remove_data (GTK_OBJECT (user_data), new_pr_list);
            }
         destroy_attrib_dlg();
         destroy_find_dlg();
         destroy_person_dlg();
         destroy_project_dlg();
         destroy_query_dlg();
         db_shutdown (curr_conn);

         /*
          * I am not _really_ sure if we need to remove it, but the 
          * documentation implies that the set will "destroy" the old
          * data, which we dont _really_ want since we never allocated
          * anything dealing with the first conn pointer...
          */
         gtk_object_remove_data (GTK_OBJECT (user_data), MY_DB_CONN);
         gtk_object_remove_data (GTK_OBJECT (user_data), ADMIN_USER);
         gtk_object_set_data (GTK_OBJECT (user_data), MY_DB_CONN, conn);
         gtk_object_set_data (GTK_OBJECT (user_data), ADMIN_USER,
             GINT_TO_POINTER (get_db_access_lvl (conn)));
         valid_new_db = TRUE;

         refresh_query_combo (GTK_WIDGET (user_data));
         refresh_project_list (user_data);
         refresh_pr_list (user_data);

         /*
          * reset the status string and the global name var
          */
         stat_string = g_string_new ("");
         g_string_sprintf (stat_string,
                           DEFAULT_STAT_STRING,
                           db_data.db_name->str);
         curr_db_name = g_string_assign (curr_db_name, db_data.db_name->str);
         appbar = gtk_object_get_data (GTK_OBJECT (user_data), APPBAR);
         g_assert (appbar != NULL);
         gnome_appbar_set_default (appbar, stat_string->str);
         g_string_free (stat_string, TRUE);
         }
      }
   clear_db_data (&db_data);

   return;
}


#ifdef PRINTING_SUPPORTED
static void
on_print (GtkWidget *widget, gpointer user_data)
{
   GtkCList   *pr_clist;
   PGconn     *conn;
   GList      *main_list = NULL;
   GList      *sel_list = NULL;
   GList      *list_iter = NULL;
   gint        i;
   GtkWidget  *msg_dlg;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   pr_clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), RESULTS_CLIST));

   /*
    * Build the main list
    */
   for (i = 0; i < pr_clist->rows; i++)
      {
      main_list = g_list_append 
         (main_list, 
          gtk_clist_get_row_data (pr_clist, i));
      }

   /*
    * If the main_list is NULL, we have nothing that can be printed.  Tell
    * the user such, and go no further.
    */
   if (main_list == NULL)
      {
      msg_dlg = gnome_message_box_new
         (_("There is nothing to print."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      return;
      }
   /*
    * Build the selection list, assume we can have multiple selections
    * whether or not we actually can.
    */
   if (pr_clist->selection)
      {
      for (list_iter = g_list_first (pr_clist->selection);
           list_iter != NULL;
           list_iter = list_iter->next)
         {
         sel_list = g_list_append 
            (sel_list,
             gtk_clist_get_row_data 
                (pr_clist, GPOINTER_TO_INT (list_iter->data)));
         }
      }

   execute_print_dlg (GTK_WIDGET (user_data), conn, main_list, sel_list);
}
#endif

static void 
on_Exit_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   GtkWidget  *msg_dlg;

   if (dirty_children (GTK_WIDGET (user_data)))
      {
      msg_dlg = gnome_message_box_new
         (_("Some items contain unsaved changes.\n"
            "If you exit, all unsaved changes will be lost.\n"
            "Do you want to exit?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 1)
         {
         return;
         }
      }

   cleanup_and_quit (GTK_WIDGET (user_data));
}


/*
 *  Signal handlers for the "Queries" menu.
 *  
 *    User...
 *    System...
 */
static void 
on_User_query_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   PGconn    *conn;
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
      (gtk_object_get_data (GTK_OBJECT (user_data), ADMIN_USER)) == 1);

   show_query_dlg (GTK_WIDGET (user_data), conn, FALSE, admin_user);
}


static void 
on_System_query_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   PGconn    *conn;
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
      (gtk_object_get_data (GTK_OBJECT (user_data), ADMIN_USER)) == 1);

   show_query_dlg (GTK_WIDGET (user_data), conn, TRUE, admin_user);
}


/*
 *  Signal handlers for the "System" menu.
 *  
 *    Projects...
 *    Users...
 *    Attributes...
 */
static void 
on_Projects_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   PGconn    *conn;
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data), ADMIN_USER)) == 1);
   show_project_dlg (GTK_WIDGET (user_data), conn, admin_user);

   return;
}


static void 
on_Users_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   PGconn    *conn;
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data), ADMIN_USER)) == 1);
   show_person_dlg (GTK_WIDGET (user_data), conn, admin_user);
   
   return;
}


static void 
on_Attributes_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   PGconn   *conn;
   gboolean  admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn);
   admin_user = (GPOINTER_TO_INT
      (gtk_object_get_data (GTK_OBJECT (user_data), ADMIN_USER)) == 1);
   show_attrib_dlg (GTK_WIDGET (user_data), conn, admin_user);
   
   return;
}


static void
on_Properties_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   show_prop_dlg (user_data);

   return;
}


static void
on_StatusRpt_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   PGconn     *conn;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn);
   execute_prj_rpt_dlg (user_data, conn, StatusRpt);

   return;
}


static void
on_VerRpt_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   PGconn  *conn;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn);
   execute_prj_rpt_dlg (user_data, conn, VersionRpt);

   return;
}


static void
on_ChangeLog_activate (GtkMenuItem *menuitem, gpointer user_data)
{
   PGconn  *conn;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn);
   execute_prj_rpt_dlg (user_data, conn, ChangeLog);

   return;
}


/*
 *  Signal handlers for the "Help" menu.
 *  
 *    About...
 */
static void 
on_About_activate (GtkMenuItem  *menuitem, gpointer  user_data)
{
   show_about_dlg (GTK_WIDGET (user_data));
   
   return;
}


/*
 *  Other signal handlers for various widgets within the dialog.
 */
static void 
on_show_inactive_cb_toggled (GtkToggleButton *togglebutton,
                             gpointer         user_data)
{
   PGconn  *conn;
   GString *config_str;

   config_str = g_string_new ("");
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DB_INI_FILE,
                     curr_db_name->str, MAIN_SH_INACT);
   gnome_config_set_bool (config_str->str,
                          gtk_toggle_button_get_active (togglebutton));
   g_string_free (config_str, TRUE);

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   draw_project_list (GTK_WIDGET (user_data), conn);
}


static void 
on_clear_button_clicked (GtkButton  *button, gpointer  user_data)
{
   GtkCList    *clist;

   clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), PROJECT_CLIST));
   g_assert (clist != NULL);

   gtk_clist_unselect_all (clist);
}


static void 
on_refresh_button_clicked (GtkButton  *button, gpointer  user_data)
{
   PGconn *conn;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   draw_pr_list (GTK_WIDGET (user_data), conn);
}


static void 
on_exit_button_clicked (GtkButton  *button, gpointer   user_data)
{
   GtkWidget *msg_dlg;

   if (dirty_children (GTK_WIDGET (user_data)))
      {
      msg_dlg = gnome_message_box_new
         (_("Some items contain unsaved changes.\n"
            "If you exit, all unsaved changes will be lost.\n"
            "Do you want to exit?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 1)
         {
         return;
         }
      }

   cleanup_and_quit (GTK_WIDGET (user_data));
}


static gboolean 
on_main_delete_event (GtkWidget *widget,
                      GdkEvent  *event,
                      gpointer   user_data)
{
  return FALSE;
}


static void 
on_main_destroy (GtkObject  *object, gpointer  user_data)
{
   cleanup_and_quit (GTK_WIDGET (object));
}


static void 
on_combo_sel_changed (GtkWidget  *widget, gpointer  user_data)
{
   PGconn    *conn;
   GList     *tmp_list;
   gint       pk;
   GString   *config_str;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   tmp_list = (GList *)gtk_object_get_data (GTK_OBJECT (widget->parent),
                                            QUERY_LIST);
   if (tmp_list != NULL)
      {
      pk = find_pk_from_name (tmp_list, 
                              gtk_entry_get_text (GTK_ENTRY (widget)));
      config_str = g_string_new ("");
      g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DB_INI_FILE,
                        curr_db_name->str, MAIN_QUERY);
      gnome_config_set_int (config_str->str, pk);
      g_string_free (config_str, TRUE);
      }

   draw_pr_list (GTK_WIDGET (user_data), conn);
}


static void
on_results_click_col (GtkCList   *clist,
                      gint        column,
                      gpointer    user_data)
{
   PGconn  *conn;
   GList   *list = NULL;
   GList   *iter;
   gint     new_col = column + 1;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   list = gtk_object_get_data (GTK_OBJECT (clist), ORDER_BY_LIST);
   /*
    * The basic idea is:
    *   Make the current column the main "order by", retaining other
    *   columns further down in the list.
    *   If the current column already is the main "order by", invert
    *   the order for it.
    *   
    */
   if (list)
      {
      gtk_object_remove_data (GTK_OBJECT (clist), ORDER_BY_LIST);

      /*
       * Scan the list, invert the selection if it already exists.
       */
      for (iter = g_list_first (list); iter != NULL; iter = iter->next)
         {
         if (abs (GPOINTER_TO_INT (iter->data)) == new_col)
            {
            new_col = -GPOINTER_TO_INT (iter->data);
            }
         }

      /*
       * Remove any existing instances of the column from the list.
       */
      list = g_list_remove (list, GINT_TO_POINTER (new_col));
      list = g_list_remove (list, GINT_TO_POINTER (-new_col));
      }
   
   list = g_list_prepend (list, GINT_TO_POINTER (new_col));
   gtk_object_set_data (GTK_OBJECT (clist), ORDER_BY_LIST, list);

   draw_pr_list (GTK_WIDGET (user_data), conn);
}


static void
on_results_sel_row (GtkWidget      *widget,
                    gint            row,
                    gint            column,
                    GdkEventButton *event,
                    gpointer        user_data)
{
   PGconn   *conn;
   GtkCList *pr_clist = GTK_CLIST (widget);
   gint      pk_num;

   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            conn = gtk_object_get_data (GTK_OBJECT (user_data),
                                        MY_DB_CONN);
            g_assert (conn != NULL);

            pk_num = GPOINTER_TO_INT 
               (gtk_clist_get_row_data 
                  (pr_clist, 
                   GPOINTER_TO_INT (pr_clist->selection->data)));

            raise_or_create_pr_update_dlg (GTK_WIDGET (user_data),
                                           conn, pk_num);

            break;

         default:
            break;
         }
      }

   return;
}


static gboolean
on_key_press_event (GtkWidget   *widget,
                    GdkEventKey *event,
                    gpointer     user_data)
{
   PGconn *conn;
   GList  *sel;
   gint    pk_num;

   if (event != NULL)
      {
      switch (event->keyval)
         {
         case GDK_KP_Enter:
         case GDK_ISO_Enter:
         case GDK_3270_Enter:
         case GDK_Return:
            sel = GTK_CLIST (widget)->selection;
            if (sel != NULL)
               {
               conn = gtk_object_get_data (GTK_OBJECT (user_data),
                                           MY_DB_CONN);
               g_assert (conn != NULL);

               pk_num = GPOINTER_TO_INT 
                  (gtk_clist_get_row_data 
                     (GTK_CLIST (widget), 
                      GPOINTER_TO_INT (sel->data)));

               raise_or_create_pr_update_dlg (GTK_WIDGET (user_data),
                                              conn, pk_num);
               }
            break;
         
         default:
            break;
         }
      }

   return FALSE;
}


static gboolean
on_button_press_event (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        user_data)
{
   gint       row;
   gint       col;
   gint       val;
   GtkWidget *popup;
   GtkWidget *main_win = GTK_WIDGET (user_data);

   static GnomeUIInfo popup_menu[] =
      {
      GNOMEUIINFO_ITEM_NONE (N_("_Edit Problem Report..."),
                             N_("Edit the selected problem report."),
                             on_modify),
#ifdef PRINTING_SUPPORTED
      GNOMEUIINFO_MENU_PRINT_ITEM (on_print, NULL),
#endif
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Problem Report..."),
                                 N_("Create a new problem report."),
                                 on_new_pr, NULL),
      GNOMEUIINFO_END
      };

   if (event)
      {
      switch (event->button)
         {
         case 1:
         case 2:
            break;
         case 3:
            switch (event->type)
               {
               case GDK_BUTTON_PRESS:
                  val = gtk_clist_get_selection_info (GTK_CLIST (widget),
                                                      event->x,
                                                      event->y,
                                                      &row, 
                                                      &col);
                  popup = gnome_popup_menu_new (popup_menu);
                  if (val)
                     {
                     gtk_clist_select_row (GTK_CLIST (widget), row, col);
                     }
                  else
                     {
                     gtk_widget_set_sensitive (popup_menu[0].widget,
                                               FALSE);
                     }

                  gnome_popup_menu_do_popup_modal (popup,
                                                   NULL,
                                                   NULL,
                                                   event,
                                                   main_win);
                  gtk_widget_destroy (popup);
                  break;
               default:
                  break;
               }
         default:
            break;
         }
      } 

   return FALSE;
}


GtkWidget * 
create_main_win (PGconn *conn)
{
  GtkWidget        *main_win;
  GtkWidget        *dock1;
  GtkWidget        *vbox1;
  GtkWidget        *table1;
  GtkWidget        *project_lbl;
  GtkWidget        *query_lbl;
  GtkWidget        *scrolledwindow3;
  GtkWidget        *project_clist;
  GtkWidget        *query_combo;
  GtkWidget        *show_inactive_cb;
  GtkWidget        *clear_button;
  GtkWidget        *label3;
  GtkWidget        *scrolledwindow2;
  GtkWidget        *results_clist;
  GtkWidget        *new_pr_button;
  GtkWidget        *mod_pr_button;
  GtkWidget        *find_pr_button;
  GtkWidget        *refresh_button;
#ifdef PRINTING_SUPPORTED
  GtkWidget        *print_button;
#endif
  GtkWidget        *exit_button;
  GtkWidget        *handlebox;
  GtkWidget        *toolbar;
  GtkWidget        *tmp_toolbar_icon;
  GtkWidget        *appbar;
  GString          *stat_string;
  GtkTooltips      *tooltips;

  GString          *cfg;
  gboolean          is_def;
  gint              x;
  gint              y;

  /*
   * See the "Maint Alert!!" below for stuff that will need to change if we
   * change the structure of the menus.
   */
  static GnomeUIInfo pr_menu_uiinfo[] =
     {
     GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Problem Report..."),
                                N_("Create a new problem report."),
                                on_new_pr, NULL),
     GNOMEUIINFO_ITEM_NONE (N_("_Edit Problem Report..."),
                            N_("Edit the selected problem report."),
                            on_modify),
     GNOMEUIINFO_ITEM_NONE
        (N_("_Find Problem Reports..."),
         N_("Find problem reports based on user defined criteria."),
         on_find),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_ITEM_NONE
        (N_("_Login..."),
         N_("Log in to a different PRepS database."),
         on_login),
#ifdef PRINTING_SUPPORTED
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_MENU_PRINT_ITEM (on_print, NULL),
#endif
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_MENU_EXIT_ITEM (on_Exit_activate, NULL),
     GNOMEUIINFO_END
     };

  static GnomeUIInfo query_submenu_uiinfo[] = 
     {
     GNOMEUIINFO_ITEM_NONE
        (N_("_User..."),
         N_("Create or modify the queries defined by the current user."),
         on_User_query_activate),
     GNOMEUIINFO_ITEM_NONE 
        (N_("_System..."),
         N_("Create or modify the queries defined for all users of "
            "the system."),
         on_System_query_activate),
     GNOMEUIINFO_END
     };

  static GnomeUIInfo admin_menu_uiinfo[] = 
     {
     GNOMEUIINFO_ITEM_NONE
        (N_("_Projects..."),
         N_("Create new projects, or modify the properties of "
            "existing projects."),
         on_Projects_activate),
     GNOMEUIINFO_ITEM_NONE
        (N_("_Users..."),
         N_("Create new users, or modify the information for "
            "existing users."),
         on_Users_activate),
     GNOMEUIINFO_ITEM_NONE
        (N_("_Attributes..."),
         N_("Create new problem report attributes (severity, status, "
            "problem type), or modify the existing attributes."),
         on_Attributes_activate),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_SUBTREE_HINT
        (N_("_Queries"),
         N_("Administer user and system level queries."), 
         query_submenu_uiinfo),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_ITEM_NONE
        (N_("P_roperties..."),
         N_("Set the properties that determine PRepS behavior"),
         on_Properties_activate),
     GNOMEUIINFO_END
     };

  static GnomeUIInfo rep_menu_uiinfo[] =
     {
     GNOMEUIINFO_ITEM_NONE
        (N_("PRs by _Status..."),
         N_("Generate an HTML report showing the problem reports "
            "grouped by status."),
         on_StatusRpt_activate),
     GNOMEUIINFO_ITEM_NONE
        (N_("PRs by _Project Version..."),
         N_("Generate an HTML report showing the problem reports "
            "grouped by project versions."),
         on_VerRpt_activate),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_ITEM_NONE
        (N_("ChangeLog..."),
         N_("Generate a text file ChangeLog template for a selected "
            " project."),
         on_ChangeLog_activate),
     GNOMEUIINFO_END
     };

  static GnomeUIInfo help_menu_uiinfo[] =
     {
     GNOMEUIINFO_HELP ("PRepS"),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_MENU_ABOUT_ITEM (on_About_activate, NULL),
     GNOMEUIINFO_END
     };

  static GnomeUIInfo menubar_uiinfo[] =
     {
     GNOMEUIINFO_SUBTREE (N_("_Problem Reports"), pr_menu_uiinfo),
     GNOMEUIINFO_SUBTREE (N_("_Administration"), admin_menu_uiinfo),
     GNOMEUIINFO_SUBTREE (N_("_Reports"), rep_menu_uiinfo),
     GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
     GNOMEUIINFO_END
     };


  tooltips = gtk_tooltips_new ();

  main_win = gnome_app_new (PACKAGE, _("PRepS - Problem Reporting System"));
  gtk_object_set_data (GTK_OBJECT (main_win), MAIN_WIN, main_win);
  gtk_container_border_width (GTK_CONTAINER (main_win), 0);
  gtk_window_set_policy (GTK_WINDOW (main_win), FALSE, TRUE, FALSE);
  gtk_signal_connect (GTK_OBJECT (main_win), "show",
                      GTK_SIGNAL_FUNC (on_main_win_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (main_win), "delete_event",
                      GTK_SIGNAL_FUNC (on_main_delete_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (main_win), "destroy",
                      GTK_SIGNAL_FUNC (on_main_destroy),
                      NULL);

  gtk_object_set_data (GTK_OBJECT (main_win), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (main_win), ADMIN_USER,
        GINT_TO_POINTER (get_db_access_lvl (conn)));

  /*
   * Since this stuff is set by the system, and not the user, I only
   * check the last "is_def" for each set.  Since the files are editable
   * by the user, it may not be valid in all cases, but it will still
   * work.
   */
  cfg = g_string_new ("");
  g_string_sprintf (cfg, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                    MAIN_WIN_SEC, X_POS, "0");
  x = gnome_config_get_int_with_default (cfg->str, &is_def);
  g_string_sprintf (cfg, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                    MAIN_WIN_SEC, Y_POS, "0");
  y = gnome_config_get_int_with_default (cfg->str, &is_def);
  if (!is_def)
     {
     gtk_widget_set_uposition (main_win, x, y);
     }
  g_string_sprintf (cfg, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                    MAIN_WIN_SEC, X_SIZE, "610");
  x = gnome_config_get_int_with_default (cfg->str, &is_def);
  g_string_sprintf (cfg, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                    MAIN_WIN_SEC, Y_SIZE, "360");
  y = gnome_config_get_int_with_default (cfg->str, &is_def);
  gtk_window_set_default_size (GTK_WINDOW (main_win), x, y);

  /*
   * Maint Alert!!!
   *
   * Now that we have the main window, pass it as user data to the menu
   * handling functions.  Note that if this menu stucture changes above
   * we will need to change this also.
   */
  pr_menu_uiinfo[0].user_data = main_win;
  pr_menu_uiinfo[1].user_data = main_win;
  pr_menu_uiinfo[2].user_data = main_win;
  pr_menu_uiinfo[4].user_data = main_win;
  pr_menu_uiinfo[6].user_data = main_win;
  /*
   * Note that if printing is not supported, exit will be in spot 6.  If 
   * printing is supported, exit is moved to slot 8.   Thus we always want
   * slot 6 initialized (it will either be exit or print), and slot 8 only
   * when printing supported (it will not even exist otherwise).
   */
#ifdef PRINTING_SUPPORTED
  pr_menu_uiinfo[8].user_data = main_win;
#endif

  admin_menu_uiinfo[0].user_data = main_win;
  admin_menu_uiinfo[1].user_data = main_win;
  admin_menu_uiinfo[2].user_data = main_win;
  admin_menu_uiinfo[6].user_data = main_win;

  query_submenu_uiinfo[0].user_data = main_win;
  query_submenu_uiinfo[1].user_data = main_win;

  rep_menu_uiinfo[0].user_data = main_win;
  rep_menu_uiinfo[1].user_data = main_win;
  rep_menu_uiinfo[3].user_data = main_win;

  help_menu_uiinfo[2].user_data = main_win;

  dock1 = GNOME_APP (main_win)->dock;
  gtk_widget_ref (dock1);
  gtk_object_set_data_full (GTK_OBJECT (main_win), "dock1", dock1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dock1);
 
  gnome_app_create_menus (GNOME_APP (main_win), menubar_uiinfo);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gnome_app_set_contents (GNOME_APP (main_win), vbox1);

  handlebox = gtk_handle_box_new ();
  gtk_widget_show (handlebox);
  gtk_box_pack_start (GTK_BOX (vbox1), handlebox, FALSE, FALSE, 0);

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_show (toolbar);
  gtk_container_add (GTK_CONTAINER (handlebox), toolbar);

  tmp_toolbar_icon = create_pixmap (main_win, "new.xpm");
  if (tmp_toolbar_icon == NULL)
     {
     g_error (_("Failed to create pixmap new.xpm"));
     }
  new_pr_button = gtk_toolbar_append_element
     (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_CHILD_BUTTON,
      NULL, _("New PR"), NULL, NULL, tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (new_pr_button);
  gtk_signal_connect (GTK_OBJECT (new_pr_button), "clicked",
                      GTK_SIGNAL_FUNC (on_new_pr),
                      main_win);
  gtk_tooltips_set_tip (tooltips, new_pr_button, NEW_STRING, NULL);

  tmp_toolbar_icon = create_pixmap (main_win, "edit.xpm");
  if (tmp_toolbar_icon == NULL)
     {
     g_error (_("Failed to create pixmap edit.xpm"));
     }
  mod_pr_button = gtk_toolbar_append_element
     (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_CHILD_BUTTON,
      NULL, _("Edit PR"), NULL, NULL, tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mod_pr_button);
  gtk_signal_connect (GTK_OBJECT (mod_pr_button), "clicked",
                      GTK_SIGNAL_FUNC (on_modify),
                      main_win);
  gtk_tooltips_set_tip (tooltips, mod_pr_button, EDIT_STRING, NULL);

  tmp_toolbar_icon = create_pixmap (main_win, "find.xpm");
  if (tmp_toolbar_icon == NULL)
     {
     g_error (_("Failed to create pixmap find.xpm"));
     }
  find_pr_button = gtk_toolbar_append_element
     (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_CHILD_BUTTON,
      NULL, _("Find PRs"), NULL, NULL, tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (find_pr_button);
  gtk_signal_connect (GTK_OBJECT (find_pr_button), "clicked",
                      GTK_SIGNAL_FUNC (on_find),
                      main_win);
  gtk_tooltips_set_tip (tooltips, find_pr_button, FIND_STRING, NULL);

#ifdef PRINTING_SUPPORTED
  tmp_toolbar_icon = create_pixmap (main_win, "print.xpm");
  if (tmp_toolbar_icon == NULL)
     {
     g_error (_("Failed to create pixmap print.xpm"));
     }
  print_button = gtk_toolbar_append_element
     (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_CHILD_BUTTON,
      NULL, _("Print"), NULL, NULL, tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (print_button);
  gtk_signal_connect (GTK_OBJECT (print_button), "clicked",
                      GTK_SIGNAL_FUNC (on_print),
                      main_win);
  gtk_tooltips_set_tip (tooltips, print_button, PRINT_STRING, NULL);
#endif

  tmp_toolbar_icon = create_pixmap (main_win, "refresh.xpm");
  if (tmp_toolbar_icon == NULL)
     {
     g_error (_("Failed to create pixmap refresh.xpm"));
     }
  refresh_button = gtk_toolbar_append_element
     (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_CHILD_BUTTON,
      NULL, "Refresh", NULL, NULL, tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (refresh_button);
  gtk_signal_connect (GTK_OBJECT (refresh_button), "clicked",
                      GTK_SIGNAL_FUNC (on_refresh_button_clicked),
                      main_win);
  gtk_tooltips_set_tip (tooltips, refresh_button, REFRESH_STRING, NULL);

  tmp_toolbar_icon = create_pixmap (main_win, "exit.xpm");
  if (tmp_toolbar_icon == NULL)
     {
     g_error (_("Failed to create pixmap exit.xpm"));
     }
  exit_button = gtk_toolbar_append_element
     (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_CHILD_BUTTON,
      NULL, "Exit", NULL, NULL, tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (exit_button);
  gtk_signal_connect (GTK_OBJECT (exit_button), "clicked",
                      GTK_SIGNAL_FUNC (on_exit_button_clicked),
                      main_win);
  gtk_tooltips_set_tip (tooltips, exit_button, EXIT_STRING, NULL);

  table1 = gtk_table_new (4, 2, FALSE);
  gtk_object_set_data (GTK_OBJECT (main_win), "table1", table1);
  gtk_container_border_width (GTK_CONTAINER (table1), 5);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, FALSE, FALSE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

  project_lbl = gtk_label_new (_("Projects"));
  gtk_object_set_data (GTK_OBJECT (main_win), "project_lbl", project_lbl);
  gtk_widget_show (project_lbl);
  gtk_table_attach (GTK_TABLE (table1), project_lbl, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_misc_set_alignment (GTK_MISC (project_lbl), 0, 0.5);

  query_lbl = gtk_label_new (_("Query"));
  gtk_object_set_data (GTK_OBJECT (main_win), "query_lbl", query_lbl);
  gtk_widget_show (query_lbl);
  gtk_table_attach (GTK_TABLE (table1), query_lbl, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_misc_set_alignment (GTK_MISC (query_lbl), 0, 0.5);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main_win), 
                       "scrolledwindow3", 
                       scrolledwindow3);
  gtk_widget_show (scrolledwindow3);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow3, 0, 1, 1, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  project_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (main_win), PROJECT_CLIST, project_clist);
  gtk_widget_show (project_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), project_clist);
  gtk_tooltips_set_tip (tooltips, project_clist, 
                        _("Select the projects that you want to query.  Selecting no projects will query all projects displayed."), 
                        NULL);
  gtk_clist_set_column_width (GTK_CLIST (project_clist), 0, 80);
  gtk_clist_set_selection_mode (GTK_CLIST (project_clist), 
                                GTK_SELECTION_MULTIPLE);
  gtk_clist_column_titles_hide (GTK_CLIST (project_clist));

  query_combo = gtk_combo_new ();
  gtk_object_set_data (GTK_OBJECT (main_win), QUERY_COMBO, query_combo);
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (query_combo)->entry),
                          FALSE);
  gtk_widget_show (query_combo);
  gtk_table_attach (GTK_TABLE (table1), query_combo, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);

  show_inactive_cb = gtk_check_button_new_with_label 
     (_("Show Inactive Projects"));
  gtk_object_set_data (GTK_OBJECT (main_win), 
                       SHOW_INACTIVE_CB, 
                       show_inactive_cb);
  gtk_widget_show (show_inactive_cb);
  gtk_table_attach (GTK_TABLE (table1), show_inactive_cb, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);

  clear_button = gtk_button_new_with_label (_("Clear Project Selections"));
  gtk_object_set_data (GTK_OBJECT (main_win), CLEAR_BUTTON, clear_button);
  gtk_widget_show (clear_button);
  gtk_table_attach (GTK_TABLE (table1), clear_button, 1, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_tooltips_set_tip (tooltips, clear_button, _("Clearing the project selections will make the query execute against all projects that are displayed."), NULL);
  gtk_signal_connect (GTK_OBJECT (clear_button), "clicked",
                      GTK_SIGNAL_FUNC (on_clear_button_clicked),
                      main_win);

  label3 = gtk_label_new (_("Selected Problem Reports"));
  gtk_object_set_data (GTK_OBJECT (main_win), "label3", label3);
  gtk_misc_set_padding (GTK_MISC (label3), 5, 0);
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (vbox1), label3, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 1.0);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main_win), 
                       "scrolledwindow2", 
                       scrolledwindow2);
  gtk_widget_show (scrolledwindow2);
  gtk_container_border_width (GTK_CONTAINER (scrolledwindow2), 5);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow2, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  results_clist = GTK_WIDGET (create_results_clist ());
  gtk_object_set_data (GTK_OBJECT (main_win), RESULTS_CLIST, results_clist);
  gtk_widget_show (results_clist);
  gtk_clist_column_titles_show (GTK_CLIST (results_clist));
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), results_clist);
  gtk_signal_connect (GTK_OBJECT (results_clist), "select_row",
                      GTK_SIGNAL_FUNC (on_results_sel_row),
                      main_win);
  gtk_signal_connect (GTK_OBJECT (results_clist), "click_column",
                      GTK_SIGNAL_FUNC (on_results_click_col),
                      main_win);
  gtk_signal_connect (GTK_OBJECT (results_clist), "key_press_event",
                      GTK_SIGNAL_FUNC (on_key_press_event),
                      main_win);
  gtk_signal_connect (GTK_OBJECT (results_clist), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press_event),
                      main_win);

  appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_object_set_data (GTK_OBJECT (main_win), APPBAR, appbar);
  gtk_widget_show (appbar);
  gnome_app_set_statusbar (GNOME_APP (main_win), appbar);
  gnome_app_install_menu_hints (GNOME_APP (main_win), menubar_uiinfo);
  
  stat_string = g_string_new ("");
  g_string_sprintf (stat_string, DEFAULT_STAT_STRING, curr_db_name->str);
  gnome_appbar_set_default (GNOME_APPBAR (appbar), stat_string->str);
  g_string_free (stat_string, TRUE);

  gtk_object_set_data (GTK_OBJECT (main_win), "tooltips", tooltips);

  return main_win;
}
